#line 2 "libemilua.so.0.11.5.p/bc2_mutex.cpp"
#include <cstring>

namespace emilua::gperf::detail { using std::size_t; using std::strcmp; }
#line 6 "libemilua.so.0.11.5.p/bc2_mutex.cpp"


#line 9 "libemilua.so.0.11.5.p/bc2_mutex.cpp"

#include <fmt/format.h>

#include <emilua/mutex.hpp>

#line 15 "libemilua.so.0.11.5.p/bc2_mutex.cpp"
namespace emilua {


static int mutex_lock(lua_State* L)
{
    auto handle = static_cast<mutex_handle*>(lua_touserdata(L, 1));
    if (!handle || !lua_getmetatable(L, 1)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }
    rawgetp(L, LUA_REGISTRYINDEX, &mutex_mt_key);
    if (!lua_rawequal(L, -1, -2)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }

    auto& vm_ctx = handle->vm_ctx;
    EMILUA_CHECK_SUSPEND_ALLOWED_ASSUMING_INTERRUPTION_DISABLED(vm_ctx, L);

    if (handle->locked) {
        handle->pending.emplace_back(vm_ctx.current_fiber());
        return lua_yield(L, 0);
    }

    handle->locked = true;
    return 0;
}

static int mutex_try_lock(lua_State* L)
{
    auto handle = static_cast<mutex_handle*>(lua_touserdata(L, 1));
    if (!handle || !lua_getmetatable(L, 1)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }
    rawgetp(L, LUA_REGISTRYINDEX, &mutex_mt_key);
    if (!lua_rawequal(L, -1, -2)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }

    if (handle->locked) {
        lua_pushboolean(L, 0);
        return 1;
    }

    handle->locked = true;
    lua_pushboolean(L, 1);
    return 1;
}

static int mutex_unlock(lua_State* L)
{
    auto handle = static_cast<mutex_handle*>(lua_touserdata(L, 1));
    if (!handle || !lua_getmetatable(L, 1)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }
    rawgetp(L, LUA_REGISTRYINDEX, &mutex_mt_key);
    if (!lua_rawequal(L, -1, -2)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }

    if (!handle->locked) {
        push(L, std::errc::operation_not_permitted);
        return lua_error(L);
    }

    if (handle->pending.size() == 0) {
        handle->locked = false;
        return 0;
    }

    auto vm_ctx = handle->vm_ctx.shared_from_this();
    auto next = handle->pending.front();
    handle->pending.pop_front();
    vm_ctx->strand().post([vm_ctx,next]() {
        vm_ctx->fiber_resume(
            next, hana::make_set(vm_context::options::skip_clear_interrupter));
    }, std::allocator<void>{});
    return 0;
}

} // namespace emilua

#line 102 "libemilua.so.0.11.5.p/bc2_mutex.cpp"
namespace emilua::gperf::detail {
namespace {
/* C++ code produced by gperf version 3.1 */
/* Command-line: /usr/bin/gperf --language=C++ --enum --readonly-tables --struct-type --initializer-suffix=',{}' --class-name=Perfect_Hash_1  */
/* Computed positions: -k'' */

struct word_type_1 { const char* name; int (*action)(lua_State*); };
/* maximum key range = 5, duplicates = 0 */

class Perfect_Hash_1
{
private:
  static inline unsigned int hash (const char *str, size_t len);
public:
  static const struct word_type_1 *in_word_set (const char *str, size_t len);
};

inline /*ARGSUSED*/
unsigned int
Perfect_Hash_1::hash (const char *str, size_t len)
{
  return len;
}

const struct word_type_1 *
Perfect_Hash_1::in_word_set (const char *str, size_t len)
{
  enum
    {
      TOTAL_KEYWORDS = 3,
      MIN_WORD_LENGTH = 4,
      MAX_WORD_LENGTH = 8,
      MIN_HASH_VALUE = 4,
      MAX_HASH_VALUE = 8
    };

  static const struct word_type_1 wordlist[] =
    {
      {"",{}}, {"",{}}, {"",{}}, {"",{}},
      {"lock", 
            [](lua_State* L) -> int {
                lua_pushcfunction(L, mutex_lock);
                return 1;
            }},
      {"",{}},
      {"unlock", 
            [](lua_State* L) -> int {
                lua_pushcfunction(L, mutex_unlock);
                return 1;
            }},
      {"",{}},
      {"try_lock", 
            [](lua_State* L) -> int {
                lua_pushcfunction(L, mutex_try_lock);
                return 1;
            }}
    };

  if (len <= MAX_WORD_LENGTH && len >= MIN_WORD_LENGTH)
    {
      unsigned int key = hash (str, len);

      if (key <= MAX_HASH_VALUE)
        {
          const char *s = wordlist[key].name;

          if (*str == *s && !strcmp (str + 1, s + 1))
            return &wordlist[key];
        }
    }
  return 0;
}

} // namespace
} // namespace emilua::gperf::detail
#line 1 "../src/mutex.cpp"
// Copyright (c) 2020 Vinícius dos Santos Oliveira
// SPDX-License-Identifier: MIT OR BSL-1.0

#line 8 "../src/mutex.cpp"


namespace emilua {

char mutex_key;
char mutex_mt_key;

inline mutex_handle::mutex_handle(vm_context& vm_ctx)
    : vm_ctx{vm_ctx}
{}

inline mutex_handle::~mutex_handle()
{
    constexpr auto spec{FMT_STRING(
        "No scheduled fibers remaining to unlock mutex {}"
    )};
    if (pending.size() != 0)
        vm_ctx.notify_deadlock(fmt::format(spec, static_cast<void*>(this)));
}

#line 110 "../src/mutex.cpp"


static int mutex_mt_index(lua_State* L)
{
    auto key = tostringview(L, 2);
    return 
#line 210 "libemilua.so.0.11.5.p/bc2_mutex.cpp"
 ::emilua::gperf::detail::value_or(::emilua::gperf::detail::Perfect_Hash_1::in_word_set((key).data(), (key).size()), [](lua_State* L) -> int {
            push(L, errc::bad_index, "index", 2);
            return lua_error(L);
        })
#line 139 "../src/mutex.cpp"
(L);
}

static int mutex_new(lua_State* L)
{
    auto& vm_ctx = get_vm_context(L);
    auto buf = static_cast<mutex_handle*>(
        lua_newuserdata(L, sizeof(mutex_handle))
    );
    rawgetp(L, LUA_REGISTRYINDEX, &mutex_mt_key);
    setmetatable(L, -2);
    new (buf) mutex_handle{vm_ctx};
    return 1;
}

void init_mutex_module(lua_State* L)
{
    lua_pushlightuserdata(L, &mutex_key);
    lua_newtable(L);
    {
        lua_createtable(L, /*narr=*/0, /*nrec=*/3);

        lua_pushliteral(L, "__metatable");
        lua_pushliteral(L, "mutex");
        lua_rawset(L, -3);

        lua_pushliteral(L, "__index");
        lua_pushcfunction(
            L,
            [](lua_State* L) -> int {
                auto key = tostringview(L, 2);
                if (key == "new") {
                    lua_pushcfunction(L, mutex_new);
                    return 1;
                } else {
                    push(L, errc::bad_index, "index", 2);
                    return lua_error(L);
                }
            });
        lua_rawset(L, -3);

        lua_pushliteral(L, "__newindex");
        lua_pushcfunction(
            L,
            [](lua_State* L) -> int {
                push(L, std::errc::operation_not_permitted);
                return lua_error(L);
            });
        lua_rawset(L, -3);
    }
    setmetatable(L, -2);
    lua_rawset(L, LUA_REGISTRYINDEX);

    lua_pushlightuserdata(L, &mutex_mt_key);
    {
        lua_createtable(L, /*narr=*/0, /*nrec=*/3);

        lua_pushliteral(L, "__metatable");
        lua_pushliteral(L, "mutex");
        lua_rawset(L, -3);

        lua_pushliteral(L, "__index");
        lua_pushcfunction(L, mutex_mt_index);
        lua_rawset(L, -3);

        lua_pushliteral(L, "__gc");
        lua_pushcfunction(L, finalizer<mutex_handle>);
        lua_rawset(L, -3);
    }
    lua_rawset(L, LUA_REGISTRYINDEX);
}

} // namespace emilua
