#line 2 "libemilua.so.0.11.5.p/bc2_tls.cpp"
#include <cstring>

namespace emilua::gperf::detail { using std::size_t; using std::strcmp; }
#line 6 "libemilua.so.0.11.5.p/bc2_tls.cpp"


#line 9 "libemilua.so.0.11.5.p/bc2_tls.cpp"

#include <boost/smart_ptr/local_shared_ptr.hpp>
#include <boost/scope_exit.hpp>

#include <emilua/async_base.hpp>
#include <emilua/filesystem.hpp>
#include <emilua/byte_span.hpp>
#include <emilua/tls.hpp>
#include <emilua/ip.hpp>

#if EMILUA_CONFIG_USE_STANDALONE_ASIO
#include <asio/ssl.hpp>
#else // EMILUA_CONFIG_USE_STANDALONE_ASIO
#include <boost/asio/ssl.hpp>
#endif // EMILUA_CONFIG_USE_STANDALONE_ASIO

#line 26 "libemilua.so.0.11.5.p/bc2_tls.cpp"
namespace emilua {


static char socket_client_handshake_key;
static char socket_server_handshake_key;
static char tls_socket_read_some_key;
static char tls_socket_write_some_key;
static char default_tls_context_key;

struct context_password_callback
{
    struct resource
    {
        resource(vm_context& vm_ctx, int ref)
            : vm_ctx{vm_ctx.weak_from_this()}
            , ref{ref}
        {
            assert(ref != LUA_NOREF);
        }

        ~resource()
        {
            auto vm_ctx = this->vm_ctx.lock();
            if (!vm_ctx)
                return;

            auto executor = vm_ctx->strand();
            executor.dispatch([ref=this->ref,vm_ctx]() {
                if (!vm_ctx->valid())
                    return;

                lua_State* L = vm_ctx->async_event_thread();
                luaL_unref(L, LUA_REGISTRYINDEX, ref);
            }, std::allocator<void>{});
        }

        std::weak_ptr<vm_context> vm_ctx;
        int ref;
    };

    context_password_callback(vm_context& vm_ctx, int ref)
        : shared_resource{new resource{vm_ctx, ref}}
    {}

    std::string operator()(std::size_t max_length,
                           asio::ssl::context::password_purpose purpose)
    {
        auto vm_ctx = shared_resource->vm_ctx.lock();
        if (!vm_ctx)
            return {};

        assert(vm_ctx->strand().running_in_this_thread());
        if (!vm_ctx->valid())
            return {};

        lua_State* L = vm_ctx->async_event_thread();
        lua_rawgeti(L, LUA_REGISTRYINDEX, shared_resource->ref);
        lua_pushinteger(L, max_length);
        try {
            switch (purpose) {
            case asio::ssl::context::for_reading:
                lua_pushliteral(L, "for_reading");
                break;
            case asio::ssl::context::for_writing:
                lua_pushliteral(L, "for_writing");
            }
        } catch (...) {
            vm_ctx->notify_errmem();
            vm_ctx->close();
            return {};
        }
        int res = lua_pcall(L, 2, 1, 0);
        if (res == LUA_ERRMEM) {
            vm_ctx->notify_errmem();
            vm_ctx->close();
            return {};
        }

        BOOST_SCOPE_EXIT_ALL(&) { lua_settop(L, 0); };
        switch (res) {
        case 0:
            if (lua_type(L, -1) != LUA_TSTRING) {
        default:
                return {};
            }

            return static_cast<std::string>(tostringview(L));
        }
    }

    boost::local_shared_ptr<resource> shared_resource;
};

} // namespace emilua

#line 122 "libemilua.so.0.11.5.p/bc2_tls.cpp"
namespace emilua::gperf::detail {
namespace {
/* C++ code produced by gperf version 3.1 */
/* Command-line: /usr/bin/gperf --language=C++ --enum --readonly-tables --struct-type --initializer-suffix=',{}' --class-name=Perfect_Hash_1  */
/* Computed positions: -k'5-6,8' */

#if !((' ' == 32) && ('!' == 33) && ('"' == 34) && ('#' == 35) \
      && ('%' == 37) && ('&' == 38) && ('\'' == 39) && ('(' == 40) \
      && (')' == 41) && ('*' == 42) && ('+' == 43) && (',' == 44) \
      && ('-' == 45) && ('.' == 46) && ('/' == 47) && ('0' == 48) \
      && ('1' == 49) && ('2' == 50) && ('3' == 51) && ('4' == 52) \
      && ('5' == 53) && ('6' == 54) && ('7' == 55) && ('8' == 56) \
      && ('9' == 57) && (':' == 58) && (';' == 59) && ('<' == 60) \
      && ('=' == 61) && ('>' == 62) && ('?' == 63) && ('A' == 65) \
      && ('B' == 66) && ('C' == 67) && ('D' == 68) && ('E' == 69) \
      && ('F' == 70) && ('G' == 71) && ('H' == 72) && ('I' == 73) \
      && ('J' == 74) && ('K' == 75) && ('L' == 76) && ('M' == 77) \
      && ('N' == 78) && ('O' == 79) && ('P' == 80) && ('Q' == 81) \
      && ('R' == 82) && ('S' == 83) && ('T' == 84) && ('U' == 85) \
      && ('V' == 86) && ('W' == 87) && ('X' == 88) && ('Y' == 89) \
      && ('Z' == 90) && ('[' == 91) && ('\\' == 92) && (']' == 93) \
      && ('^' == 94) && ('_' == 95) && ('a' == 97) && ('b' == 98) \
      && ('c' == 99) && ('d' == 100) && ('e' == 101) && ('f' == 102) \
      && ('g' == 103) && ('h' == 104) && ('i' == 105) && ('j' == 106) \
      && ('k' == 107) && ('l' == 108) && ('m' == 109) && ('n' == 110) \
      && ('o' == 111) && ('p' == 112) && ('q' == 113) && ('r' == 114) \
      && ('s' == 115) && ('t' == 116) && ('u' == 117) && ('v' == 118) \
      && ('w' == 119) && ('x' == 120) && ('y' == 121) && ('z' == 122) \
      && ('{' == 123) && ('|' == 124) && ('}' == 125) && ('~' == 126))
/* The character set is not based on ISO-646.  */
#error "gperf generated tables don't work with this execution character set. Please report a bug to <bug-gperf@gnu.org>."
#endif

struct word_type_1 { const char* name; asio::ssl::context::method action; };
/* maximum key range = 46, duplicates = 0 */

class Perfect_Hash_1
{
private:
  static inline unsigned int hash (const char *str, size_t len);
public:
  static const struct word_type_1 *in_word_set (const char *str, size_t len);
};

inline unsigned int
Perfect_Hash_1::hash (const char *str, size_t len)
{
  static const unsigned char asso_values[] =
    {
      49, 49, 49, 49, 49, 49, 49, 49, 49, 49,
      49, 49, 49, 49, 49, 49, 49, 49, 49, 49,
      49, 49, 49, 49, 49, 49, 49, 49, 49, 49,
      49, 49, 49, 49, 49, 49, 49, 49, 49, 49,
      49, 49, 49, 49, 49, 49, 49, 49, 49,  0,
      10,  5, 49, 49, 49, 49, 49, 49, 49, 49,
      49, 49, 49, 49, 49, 49, 49, 49, 49, 49,
      49, 49, 49, 49, 49, 49, 49, 49, 49, 49,
      49, 49, 49, 49, 49, 49, 49, 49, 49, 49,
      49, 49, 49, 49, 49,  0, 49, 49, 49, 20,
      49, 15, 49, 49, 49, 49, 49, 49,  0, 49,
      49, 49, 49, 49, 49,  0, 49, 49,  0, 49,
      49, 49, 49, 49, 49, 49, 49, 49, 49, 49,
      49, 49, 49, 49, 49, 49, 49, 49, 49, 49,
      49, 49, 49, 49, 49, 49, 49, 49, 49, 49,
      49, 49, 49, 49, 49, 49, 49, 49, 49, 49,
      49, 49, 49, 49, 49, 49, 49, 49, 49, 49,
      49, 49, 49, 49, 49, 49, 49, 49, 49, 49,
      49, 49, 49, 49, 49, 49, 49, 49, 49, 49,
      49, 49, 49, 49, 49, 49, 49, 49, 49, 49,
      49, 49, 49, 49, 49, 49, 49, 49, 49, 49,
      49, 49, 49, 49, 49, 49, 49, 49, 49, 49,
      49, 49, 49, 49, 49, 49, 49, 49, 49, 49,
      49, 49, 49, 49, 49, 49, 49, 49, 49, 49,
      49, 49, 49, 49, 49, 49, 49, 49, 49, 49,
      49, 49, 49, 49, 49, 49
    };
  unsigned int hval = len;

  switch (hval)
    {
      default:
        hval += asso_values[static_cast<unsigned char>(str[7])];
      /*FALLTHROUGH*/
      case 7:
      case 6:
        hval += asso_values[static_cast<unsigned char>(str[5])];
      /*FALLTHROUGH*/
      case 5:
        hval += asso_values[static_cast<unsigned char>(str[4])];
      /*FALLTHROUGH*/
      case 4:
      case 3:
        break;
    }
  return hval;
}

const struct word_type_1 *
Perfect_Hash_1::in_word_set (const char *str, size_t len)
{
  enum
    {
      TOTAL_KEYWORDS = 24,
      MIN_WORD_LENGTH = 3,
      MAX_WORD_LENGTH = 13,
      MIN_HASH_VALUE = 3,
      MAX_HASH_VALUE = 48
    };

  static const struct word_type_1 wordlist[] =
    {
      {"",{}}, {"",{}}, {"",{}},
      {"tls",  asio::ssl::context::tls},
      {"",{}},
      {"tlsv1",  asio::ssl::context::tlsv1},
      {"tlsv11",  asio::ssl::context::tlsv11},
      {"",{}}, {"",{}}, {"",{}},
      {"sslv3",  asio::ssl::context::sslv3},
      {"tlsv13",  asio::ssl::context::tlsv13},
      {"tlsv1_client",  asio::ssl::context::tlsv1_client},
      {"tlsv11_server",  asio::ssl::context::tlsv11_server},
      {"",{}},
      {"sslv2",  asio::ssl::context::sslv2},
      {"tlsv12",  asio::ssl::context::tlsv12},
      {"sslv3_client",  asio::ssl::context::sslv3_client},
      {"tlsv13_server",  asio::ssl::context::tlsv13_server},
      {"",{}}, {"",{}},
      {"sslv23",  asio::ssl::context::sslv23},
      {"sslv2_client",  asio::ssl::context::sslv2_client},
      {"tlsv12_server",  asio::ssl::context::tlsv12_server},
      {"",{}},
      {"tls_server",  asio::ssl::context::tls_server},
      {"",{}},
      {"tlsv1_server",  asio::ssl::context::tlsv1_server},
      {"sslv23_server",  asio::ssl::context::sslv23_server},
      {"",{}}, {"",{}}, {"",{}},
      {"sslv3_server",  asio::ssl::context::sslv3_server},
      {"tlsv11_client",  asio::ssl::context::tlsv11_client},
      {"",{}}, {"",{}}, {"",{}},
      {"sslv2_server",  asio::ssl::context::sslv2_server},
      {"tlsv13_client",  asio::ssl::context::tlsv13_client},
      {"",{}}, {"",{}}, {"",{}}, {"",{}},
      {"tlsv12_client",  asio::ssl::context::tlsv12_client},
      {"",{}},
      {"tls_client",  asio::ssl::context::tls_client},
      {"",{}}, {"",{}},
      {"sslv23_client",  asio::ssl::context::sslv23_client}
    };

  if (len <= MAX_WORD_LENGTH && len >= MIN_WORD_LENGTH)
    {
      unsigned int key = hash (str, len);

      if (key <= MAX_HASH_VALUE)
        {
          const char *s = wordlist[key].name;

          if (*str == *s && !strcmp (str + 1, s + 1))
            return &wordlist[key];
        }
    }
  return 0;
}

} // namespace
} // namespace emilua::gperf::detail

#line 290 "libemilua.so.0.11.5.p/bc2_tls.cpp"
namespace emilua::gperf::detail {
namespace {
/* C++ code produced by gperf version 3.1 */
/* Command-line: /usr/bin/gperf --language=C++ --enum --readonly-tables --struct-type --initializer-suffix=',{}' --class-name=Perfect_Hash_2  */
/* Computed positions: -k'$' */

#if !((' ' == 32) && ('!' == 33) && ('"' == 34) && ('#' == 35) \
      && ('%' == 37) && ('&' == 38) && ('\'' == 39) && ('(' == 40) \
      && (')' == 41) && ('*' == 42) && ('+' == 43) && (',' == 44) \
      && ('-' == 45) && ('.' == 46) && ('/' == 47) && ('0' == 48) \
      && ('1' == 49) && ('2' == 50) && ('3' == 51) && ('4' == 52) \
      && ('5' == 53) && ('6' == 54) && ('7' == 55) && ('8' == 56) \
      && ('9' == 57) && (':' == 58) && (';' == 59) && ('<' == 60) \
      && ('=' == 61) && ('>' == 62) && ('?' == 63) && ('A' == 65) \
      && ('B' == 66) && ('C' == 67) && ('D' == 68) && ('E' == 69) \
      && ('F' == 70) && ('G' == 71) && ('H' == 72) && ('I' == 73) \
      && ('J' == 74) && ('K' == 75) && ('L' == 76) && ('M' == 77) \
      && ('N' == 78) && ('O' == 79) && ('P' == 80) && ('Q' == 81) \
      && ('R' == 82) && ('S' == 83) && ('T' == 84) && ('U' == 85) \
      && ('V' == 86) && ('W' == 87) && ('X' == 88) && ('Y' == 89) \
      && ('Z' == 90) && ('[' == 91) && ('\\' == 92) && (']' == 93) \
      && ('^' == 94) && ('_' == 95) && ('a' == 97) && ('b' == 98) \
      && ('c' == 99) && ('d' == 100) && ('e' == 101) && ('f' == 102) \
      && ('g' == 103) && ('h' == 104) && ('i' == 105) && ('j' == 106) \
      && ('k' == 107) && ('l' == 108) && ('m' == 109) && ('n' == 110) \
      && ('o' == 111) && ('p' == 112) && ('q' == 113) && ('r' == 114) \
      && ('s' == 115) && ('t' == 116) && ('u' == 117) && ('v' == 118) \
      && ('w' == 119) && ('x' == 120) && ('y' == 121) && ('z' == 122) \
      && ('{' == 123) && ('|' == 124) && ('}' == 125) && ('~' == 126))
/* The character set is not based on ISO-646.  */
#error "gperf generated tables don't work with this execution character set. Please report a bug to <bug-gperf@gnu.org>."
#endif

struct word_type_2 { const char* name; asio::ssl::context_base::options action; };
/* maximum key range = 16, duplicates = 0 */

class Perfect_Hash_2
{
private:
  static inline unsigned int hash (const char *str, size_t len);
public:
  static const struct word_type_2 *in_word_set (const char *str, size_t len);
};

inline unsigned int
Perfect_Hash_2::hash (const char *str, size_t len)
{
  static const unsigned char asso_values[] =
    {
      24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
      24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
      24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
      24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
      24, 24, 24, 24, 24, 24, 24, 24, 24, 10,
       5,  0, 24, 24, 24, 24, 24, 24, 24, 24,
      24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
      24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
      24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
      24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
      24, 10, 24, 24, 24, 24, 24, 24, 24, 24,
       0, 24, 24, 24, 24,  0, 24, 24, 24, 24,
      24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
      24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
      24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
      24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
      24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
      24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
      24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
      24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
      24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
      24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
      24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
      24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
      24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
      24, 24, 24, 24, 24, 24
    };
  return len + asso_values[static_cast<unsigned char>(str[len - 1])];
}

const struct word_type_2 *
Perfect_Hash_2::in_word_set (const char *str, size_t len)
{
  enum
    {
      TOTAL_KEYWORDS = 9,
      MIN_WORD_LENGTH = 8,
      MAX_WORD_LENGTH = 19,
      MIN_HASH_VALUE = 8,
      MAX_HASH_VALUE = 23
    };

  static const struct word_type_2 wordlist[] =
    {
      {"",{}}, {"",{}}, {"",{}}, {"",{}}, {"",{}}, {"",{}},
      {"",{}}, {"",{}},
      {"no_sslv3",  asio::ssl::context_base::no_sslv3},
      {"",{}},
      {"no_tlsv1_3",  asio::ssl::context_base::no_tlsv1_3},
      {"",{}}, {"",{}},
      {"no_sslv2",  asio::ssl::context_base::no_sslv2},
      {"no_compression",  asio::ssl::context_base::no_compression},
      {"no_tlsv1_2",  asio::ssl::context_base::no_tlsv1_2},
      {"",{}}, {"",{}},
      {"no_tlsv1",  asio::ssl::context_base::no_tlsv1},
      {"default_workarounds", 
                asio::ssl::context_base::default_workarounds},
      {"no_tlsv1_1",  asio::ssl::context_base::no_tlsv1_1},
      {"",{}}, {"",{}},
      {"single_dh_use",  asio::ssl::context_base::single_dh_use}
    };

  if (len <= MAX_WORD_LENGTH && len >= MIN_WORD_LENGTH)
    {
      unsigned int key = hash (str, len);

      if (key <= MAX_HASH_VALUE)
        {
          const char *s = wordlist[key].name;

          if (*str == *s && !strcmp (str + 1, s + 1))
            return &wordlist[key];
        }
    }
  return 0;
}

} // namespace
} // namespace emilua::gperf::detail

#line 420 "libemilua.so.0.11.5.p/bc2_tls.cpp"
namespace emilua::gperf::detail {
namespace {
/* C++ code produced by gperf version 3.1 */
/* Command-line: /usr/bin/gperf --language=C++ --enum --readonly-tables --struct-type --initializer-suffix=',{}' --class-name=Perfect_Hash_3  */
/* Computed positions: -k'$' */

#if !((' ' == 32) && ('!' == 33) && ('"' == 34) && ('#' == 35) \
      && ('%' == 37) && ('&' == 38) && ('\'' == 39) && ('(' == 40) \
      && (')' == 41) && ('*' == 42) && ('+' == 43) && (',' == 44) \
      && ('-' == 45) && ('.' == 46) && ('/' == 47) && ('0' == 48) \
      && ('1' == 49) && ('2' == 50) && ('3' == 51) && ('4' == 52) \
      && ('5' == 53) && ('6' == 54) && ('7' == 55) && ('8' == 56) \
      && ('9' == 57) && (':' == 58) && (';' == 59) && ('<' == 60) \
      && ('=' == 61) && ('>' == 62) && ('?' == 63) && ('A' == 65) \
      && ('B' == 66) && ('C' == 67) && ('D' == 68) && ('E' == 69) \
      && ('F' == 70) && ('G' == 71) && ('H' == 72) && ('I' == 73) \
      && ('J' == 74) && ('K' == 75) && ('L' == 76) && ('M' == 77) \
      && ('N' == 78) && ('O' == 79) && ('P' == 80) && ('Q' == 81) \
      && ('R' == 82) && ('S' == 83) && ('T' == 84) && ('U' == 85) \
      && ('V' == 86) && ('W' == 87) && ('X' == 88) && ('Y' == 89) \
      && ('Z' == 90) && ('[' == 91) && ('\\' == 92) && (']' == 93) \
      && ('^' == 94) && ('_' == 95) && ('a' == 97) && ('b' == 98) \
      && ('c' == 99) && ('d' == 100) && ('e' == 101) && ('f' == 102) \
      && ('g' == 103) && ('h' == 104) && ('i' == 105) && ('j' == 106) \
      && ('k' == 107) && ('l' == 108) && ('m' == 109) && ('n' == 110) \
      && ('o' == 111) && ('p' == 112) && ('q' == 113) && ('r' == 114) \
      && ('s' == 115) && ('t' == 116) && ('u' == 117) && ('v' == 118) \
      && ('w' == 119) && ('x' == 120) && ('y' == 121) && ('z' == 122) \
      && ('{' == 123) && ('|' == 124) && ('}' == 125) && ('~' == 126))
/* The character set is not based on ISO-646.  */
#error "gperf generated tables don't work with this execution character set. Please report a bug to <bug-gperf@gnu.org>."
#endif

struct word_type_3 { const char* name; asio::ssl::context_base::options action; };
/* maximum key range = 16, duplicates = 0 */

class Perfect_Hash_3
{
private:
  static inline unsigned int hash (const char *str, size_t len);
public:
  static const struct word_type_3 *in_word_set (const char *str, size_t len);
};

inline unsigned int
Perfect_Hash_3::hash (const char *str, size_t len)
{
  static const unsigned char asso_values[] =
    {
      24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
      24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
      24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
      24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
      24, 24, 24, 24, 24, 24, 24, 24, 24, 10,
       5,  0, 24, 24, 24, 24, 24, 24, 24, 24,
      24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
      24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
      24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
      24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
      24, 10, 24, 24, 24, 24, 24, 24, 24, 24,
       0, 24, 24, 24, 24,  0, 24, 24, 24, 24,
      24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
      24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
      24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
      24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
      24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
      24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
      24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
      24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
      24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
      24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
      24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
      24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
      24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
      24, 24, 24, 24, 24, 24
    };
  return len + asso_values[static_cast<unsigned char>(str[len - 1])];
}

const struct word_type_3 *
Perfect_Hash_3::in_word_set (const char *str, size_t len)
{
  enum
    {
      TOTAL_KEYWORDS = 9,
      MIN_WORD_LENGTH = 8,
      MAX_WORD_LENGTH = 19,
      MIN_HASH_VALUE = 8,
      MAX_HASH_VALUE = 23
    };

  static const struct word_type_3 wordlist[] =
    {
      {"",{}}, {"",{}}, {"",{}}, {"",{}}, {"",{}}, {"",{}},
      {"",{}}, {"",{}},
      {"no_sslv3",  asio::ssl::context_base::no_sslv3},
      {"",{}},
      {"no_tlsv1_3",  asio::ssl::context_base::no_tlsv1_3},
      {"",{}}, {"",{}},
      {"no_sslv2",  asio::ssl::context_base::no_sslv2},
      {"no_compression",  asio::ssl::context_base::no_compression},
      {"no_tlsv1_2",  asio::ssl::context_base::no_tlsv1_2},
      {"",{}}, {"",{}},
      {"no_tlsv1",  asio::ssl::context_base::no_tlsv1},
      {"default_workarounds", 
                asio::ssl::context_base::default_workarounds},
      {"no_tlsv1_1",  asio::ssl::context_base::no_tlsv1_1},
      {"",{}}, {"",{}},
      {"single_dh_use",  asio::ssl::context_base::single_dh_use}
    };

  if (len <= MAX_WORD_LENGTH && len >= MIN_WORD_LENGTH)
    {
      unsigned int key = hash (str, len);

      if (key <= MAX_HASH_VALUE)
        {
          const char *s = wordlist[key].name;

          if (*str == *s && !strcmp (str + 1, s + 1))
            return &wordlist[key];
        }
    }
  return 0;
}

} // namespace
} // namespace emilua::gperf::detail

#line 550 "libemilua.so.0.11.5.p/bc2_tls.cpp"
namespace emilua::gperf::detail {
namespace {
/* C++ code produced by gperf version 3.1 */
/* Command-line: /usr/bin/gperf --language=C++ --enum --readonly-tables --struct-type --initializer-suffix=',{}' --class-name=Perfect_Hash_4  */
/* Computed positions: -k'' */

struct word_type_4 { const char* name; 
            int (*action)(lua_State*, std::shared_ptr<asio::ssl::context>*); };
/* maximum key range = 1, duplicates = 0 */

class Perfect_Hash_4
{
private:
  static inline unsigned int hash (const char *str, size_t len);
public:
  static const struct word_type_4 *in_word_set (const char *str, size_t len);
};

inline /*ARGSUSED*/
unsigned int
Perfect_Hash_4::hash (const char *str, size_t len)
{
  return 0;
}

const struct word_type_4 *
Perfect_Hash_4::in_word_set (const char *str, size_t len)
{
  enum
    {
      TOTAL_KEYWORDS = 1,
      MIN_WORD_LENGTH = 22,
      MAX_WORD_LENGTH = 22,
      MIN_HASH_VALUE = 0,
      MAX_HASH_VALUE = 0
    };

  static const struct word_type_4 wordlist[] =
    {
      {"host_name_verification", 
            [](lua_State* L, std::shared_ptr<asio::ssl::context>* ctx) -> int {
                luaL_checktype(L, 3, LUA_TSTRING);
                asio::ssl::host_name_verification o{
                    static_cast<std::string>(tostringview(L, 3))};
                asio_error_code ec;
                (*ctx)->set_verify_callback(std::move(o), ec);
                if (ec) {
                    push(L, static_cast<std::error_code>(ec));
                    return lua_error(L);
                }
                return 0;
            }}
    };

  if (len <= MAX_WORD_LENGTH && len >= MIN_WORD_LENGTH)
    {
      unsigned int key = hash (str, len);

      if (key <= MAX_HASH_VALUE)
        {
          const char *s = wordlist[key].name;

          if (*str == *s && !strcmp (str + 1, s + 1))
            return &wordlist[key];
        }
    }
  return 0;
}

} // namespace
} // namespace emilua::gperf::detail

#line 623 "libemilua.so.0.11.5.p/bc2_tls.cpp"
namespace emilua::gperf::detail {
namespace {
/* C++ code produced by gperf version 3.1 */
/* Command-line: /usr/bin/gperf --language=C++ --enum --readonly-tables --struct-type --initializer-suffix=',{}' --class-name=Perfect_Hash_5  */
/* Computed positions: -k'1' */

#if !((' ' == 32) && ('!' == 33) && ('"' == 34) && ('#' == 35) \
      && ('%' == 37) && ('&' == 38) && ('\'' == 39) && ('(' == 40) \
      && (')' == 41) && ('*' == 42) && ('+' == 43) && (',' == 44) \
      && ('-' == 45) && ('.' == 46) && ('/' == 47) && ('0' == 48) \
      && ('1' == 49) && ('2' == 50) && ('3' == 51) && ('4' == 52) \
      && ('5' == 53) && ('6' == 54) && ('7' == 55) && ('8' == 56) \
      && ('9' == 57) && (':' == 58) && (';' == 59) && ('<' == 60) \
      && ('=' == 61) && ('>' == 62) && ('?' == 63) && ('A' == 65) \
      && ('B' == 66) && ('C' == 67) && ('D' == 68) && ('E' == 69) \
      && ('F' == 70) && ('G' == 71) && ('H' == 72) && ('I' == 73) \
      && ('J' == 74) && ('K' == 75) && ('L' == 76) && ('M' == 77) \
      && ('N' == 78) && ('O' == 79) && ('P' == 80) && ('Q' == 81) \
      && ('R' == 82) && ('S' == 83) && ('T' == 84) && ('U' == 85) \
      && ('V' == 86) && ('W' == 87) && ('X' == 88) && ('Y' == 89) \
      && ('Z' == 90) && ('[' == 91) && ('\\' == 92) && (']' == 93) \
      && ('^' == 94) && ('_' == 95) && ('a' == 97) && ('b' == 98) \
      && ('c' == 99) && ('d' == 100) && ('e' == 101) && ('f' == 102) \
      && ('g' == 103) && ('h' == 104) && ('i' == 105) && ('j' == 106) \
      && ('k' == 107) && ('l' == 108) && ('m' == 109) && ('n' == 110) \
      && ('o' == 111) && ('p' == 112) && ('q' == 113) && ('r' == 114) \
      && ('s' == 115) && ('t' == 116) && ('u' == 117) && ('v' == 118) \
      && ('w' == 119) && ('x' == 120) && ('y' == 121) && ('z' == 122) \
      && ('{' == 123) && ('|' == 124) && ('}' == 125) && ('~' == 126))
/* The character set is not based on ISO-646.  */
#error "gperf generated tables don't work with this execution character set. Please report a bug to <bug-gperf@gnu.org>."
#endif

struct word_type_5 { const char* name; 
            int (*action)(lua_State*, std::shared_ptr<asio::ssl::context>*); };
/* maximum key range = 17, duplicates = 0 */

class Perfect_Hash_5
{
private:
  static inline unsigned int hash (const char *str, size_t len);
public:
  static const struct word_type_5 *in_word_set (const char *str, size_t len);
};

inline unsigned int
Perfect_Hash_5::hash (const char *str, size_t len)
{
  static const unsigned char asso_values[] =
    {
      21, 21, 21, 21, 21, 21, 21, 21, 21, 21,
      21, 21, 21, 21, 21, 21, 21, 21, 21, 21,
      21, 21, 21, 21, 21, 21, 21, 21, 21, 21,
      21, 21, 21, 21, 21, 21, 21, 21, 21, 21,
      21, 21, 21, 21, 21, 21, 21, 21, 21, 21,
      21, 21, 21, 21, 21, 21, 21, 21, 21, 21,
      21, 21, 21, 21, 21, 21, 21, 21, 21, 21,
      21, 21, 21, 21, 21, 21, 21, 21, 21, 21,
      21, 21, 21, 21, 21, 21, 21, 21, 21, 21,
      21, 21, 21, 21, 21, 21, 21, 21, 21,  0,
      21, 21,  0, 21, 21, 21, 21, 21, 21, 21,
       5, 21,  0, 21, 21, 21, 21, 21, 21, 21,
      21, 21, 21, 21, 21, 21, 21, 21, 21, 21,
      21, 21, 21, 21, 21, 21, 21, 21, 21, 21,
      21, 21, 21, 21, 21, 21, 21, 21, 21, 21,
      21, 21, 21, 21, 21, 21, 21, 21, 21, 21,
      21, 21, 21, 21, 21, 21, 21, 21, 21, 21,
      21, 21, 21, 21, 21, 21, 21, 21, 21, 21,
      21, 21, 21, 21, 21, 21, 21, 21, 21, 21,
      21, 21, 21, 21, 21, 21, 21, 21, 21, 21,
      21, 21, 21, 21, 21, 21, 21, 21, 21, 21,
      21, 21, 21, 21, 21, 21, 21, 21, 21, 21,
      21, 21, 21, 21, 21, 21, 21, 21, 21, 21,
      21, 21, 21, 21, 21, 21, 21, 21, 21, 21,
      21, 21, 21, 21, 21, 21, 21, 21, 21, 21,
      21, 21, 21, 21, 21, 21
    };
  return len + asso_values[static_cast<unsigned char>(str[0])];
}

const struct word_type_5 *
Perfect_Hash_5::in_word_set (const char *str, size_t len)
{
  enum
    {
      TOTAL_KEYWORDS = 4,
      MIN_WORD_LENGTH = 4,
      MAX_WORD_LENGTH = 20,
      MIN_HASH_VALUE = 4,
      MAX_HASH_VALUE = 20
    };

  static const struct word_type_5 wordlist[] =
    {
      {"",{}}, {"",{}}, {"",{}}, {"",{}},
      {"peer", 
            [](lua_State* L, std::shared_ptr<asio::ssl::context>* ctx) -> int {
                asio_error_code ec;
                (*ctx)->set_verify_mode(asio::ssl::verify_peer, ec);
                if (ec) {
                    push(L, static_cast<std::error_code>(ec));
                    return lua_error(L);
                }
                return 0;
            }},
      {"",{}}, {"",{}}, {"",{}}, {"",{}},
      {"none", 
            [](lua_State* L, std::shared_ptr<asio::ssl::context>* ctx) -> int {
                asio_error_code ec;
                (*ctx)->set_verify_mode(asio::ssl::verify_none, ec);
                if (ec) {
                    push(L, static_cast<std::error_code>(ec));
                    return lua_error(L);
                }
                return 0;
            }},
      {"",{}},
      {"client_once", 
            [](lua_State* L, std::shared_ptr<asio::ssl::context>* ctx) -> int {
                asio_error_code ec;
                (*ctx)->set_verify_mode(asio::ssl::verify_client_once, ec);
                if (ec) {
                    push(L, static_cast<std::error_code>(ec));
                    return lua_error(L);
                }
                return 0;
            }},
      {"",{}}, {"",{}}, {"",{}}, {"",{}}, {"",{}}, {"",{}},
      {"",{}}, {"",{}},
      {"fail_if_no_peer_cert", 
            [](lua_State* L, std::shared_ptr<asio::ssl::context>* ctx) -> int {
                asio_error_code ec;
                (*ctx)->set_verify_mode(
                    asio::ssl::verify_fail_if_no_peer_cert, ec);
                if (ec) {
                    push(L, static_cast<std::error_code>(ec));
                    return lua_error(L);
                }
                return 0;
            }}
    };

  if (len <= MAX_WORD_LENGTH && len >= MIN_WORD_LENGTH)
    {
      unsigned int key = hash (str, len);

      if (key <= MAX_HASH_VALUE)
        {
          const char *s = wordlist[key].name;

          if (*str == *s && !strcmp (str + 1, s + 1))
            return &wordlist[key];
        }
    }
  return 0;
}

} // namespace
} // namespace emilua::gperf::detail

#line 784 "libemilua.so.0.11.5.p/bc2_tls.cpp"
namespace emilua {


static int context_add_certificate_authority(lua_State* L)
{
    auto ctx = static_cast<std::shared_ptr<asio::ssl::context>*>(
        lua_touserdata(L, 1)
    );
    if (!ctx || !lua_getmetatable(L, 1)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }
    rawgetp(L, LUA_REGISTRYINDEX, &tls_context_mt_key);
    if (!lua_rawequal(L, -1, -2)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }

    auto bs = static_cast<byte_span_handle*>(lua_touserdata(L, 2));
    if (!bs || !lua_getmetatable(L, 2)) {
        push(L, std::errc::invalid_argument, "arg", 2);
        return lua_error(L);
    }
    rawgetp(L, LUA_REGISTRYINDEX, &byte_span_mt_key);
    if (!lua_rawequal(L, -1, -2)) {
        push(L, std::errc::invalid_argument, "arg", 2);
        return lua_error(L);
    }

    asio_error_code ec;
    (*ctx)->add_certificate_authority(
        asio::buffer(bs->data.get(), bs->size), ec);
    if (ec) {
        push(L, ec);
        return lua_error(L);
    }
    return 0;
}

static int context_add_verify_path(lua_State* L)
{
    lua_settop(L, 2);

    auto ctx = static_cast<std::shared_ptr<asio::ssl::context>*>(
        lua_touserdata(L, 1)
    );
    if (!ctx || !lua_getmetatable(L, 1)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }
    rawgetp(L, LUA_REGISTRYINDEX, &tls_context_mt_key);
    if (!lua_rawequal(L, -1, -2)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }

    std::string path;
    try {
        auto p = static_cast<std::filesystem::path*>(lua_touserdata(L, 2));
        if (!p || !lua_getmetatable(L, 2)) {
            push(L, std::errc::invalid_argument, "arg", 2);
            return lua_error(L);
        }
        rawgetp(L, LUA_REGISTRYINDEX, &filesystem_path_mt_key);
        if (!lua_rawequal(L, -1, -2)) {
            push(L, std::errc::invalid_argument, "arg", 2);
            return lua_error(L);
        }

        path = p->string();
    } catch (const std::system_error& e) {
        push(L, e.code());
        return lua_error(L);
    } catch (const std::exception& e) {
        lua_pushstring(L, e.what());
        return lua_error(L);
    }

    asio_error_code ec;
    (*ctx)->add_verify_path(path, ec);
    if (ec) {
        push(L, ec);
        return lua_error(L);
    }
    return 0;
}

static int context_clear_options(lua_State* L)
{
    luaL_checktype(L, 2, LUA_TTABLE);

    auto ctx = static_cast<std::shared_ptr<asio::ssl::context>*>(
        lua_touserdata(L, 1)
    );
    if (!ctx || !lua_getmetatable(L, 1)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }
    rawgetp(L, LUA_REGISTRYINDEX, &tls_context_mt_key);
    if (!lua_rawequal(L, -1, -2)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }

    asio::ssl::context_base::options flags = {};
    for (int i = 1 ;; ++i) {
        lua_rawgeti(L, 2, i);
        switch (lua_type(L, -1)) {
        default:
            push(L, std::errc::invalid_argument, "arg", 2);
            return lua_error(L);
        case LUA_TNIL:
            lua_pop(L, 1);
            goto end_for;
        case LUA_TSTRING:
            break;
        }

        auto s = tostringview(L);
        lua_pop(L, 1);
        auto f =  ::emilua::gperf::detail::value_or(::emilua::gperf::detail::Perfect_Hash_2::in_word_set((s).data(), (s).size()), {});
        if (f == 0) {
            push(L, std::errc::invalid_argument, "arg", 2);
            return lua_error(L);
        }
        flags |= f;
    }
 end_for:

    asio_error_code ec;
    (*ctx)->clear_options(flags, ec);
    if (ec) {
        push(L, ec);
        return lua_error(L);
    }
    return 0;
}

static int context_load_verify_file(lua_State* L)
{
    lua_settop(L, 2);

    auto ctx = static_cast<std::shared_ptr<asio::ssl::context>*>(
        lua_touserdata(L, 1)
    );
    if (!ctx || !lua_getmetatable(L, 1)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }
    rawgetp(L, LUA_REGISTRYINDEX, &tls_context_mt_key);
    if (!lua_rawequal(L, -1, -2)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }

    std::string path;
    try {
        auto p = static_cast<std::filesystem::path*>(lua_touserdata(L, 2));
        if (!p || !lua_getmetatable(L, 2)) {
            push(L, std::errc::invalid_argument, "arg", 2);
            return lua_error(L);
        }
        rawgetp(L, LUA_REGISTRYINDEX, &filesystem_path_mt_key);
        if (!lua_rawequal(L, -1, -2)) {
            push(L, std::errc::invalid_argument, "arg", 2);
            return lua_error(L);
        }

        path = p->string();
    } catch (const std::system_error& e) {
        push(L, e.code());
        return lua_error(L);
    } catch (const std::exception& e) {
        lua_pushstring(L, e.what());
        return lua_error(L);
    }

    asio_error_code ec;
    (*ctx)->load_verify_file(path, ec);
    if (ec) {
        push(L, ec);
        return lua_error(L);
    }
    return 0;
}

static int context_set_default_verify_paths(lua_State* L)
{
    auto ctx = static_cast<std::shared_ptr<asio::ssl::context>*>(
        lua_touserdata(L, 1)
    );
    if (!ctx || !lua_getmetatable(L, 1)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }
    rawgetp(L, LUA_REGISTRYINDEX, &tls_context_mt_key);
    if (!lua_rawequal(L, -1, -2)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }

    asio_error_code ec;
    (*ctx)->set_default_verify_paths(ec);
    if (ec) {
        push(L, ec);
        return lua_error(L);
    }
    return 0;
}

static int context_set_options(lua_State* L)
{
    luaL_checktype(L, 2, LUA_TTABLE);

    auto ctx = static_cast<std::shared_ptr<asio::ssl::context>*>(
        lua_touserdata(L, 1)
    );
    if (!ctx || !lua_getmetatable(L, 1)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }
    rawgetp(L, LUA_REGISTRYINDEX, &tls_context_mt_key);
    if (!lua_rawequal(L, -1, -2)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }

    asio::ssl::context_base::options flags = {};
    for (int i = 1 ;; ++i) {
        lua_rawgeti(L, 2, i);
        switch (lua_type(L, -1)) {
        default:
            push(L, std::errc::invalid_argument, "arg", 2);
            return lua_error(L);
        case LUA_TNIL:
            lua_pop(L, 1);
            goto end_for;
        case LUA_TSTRING:
            break;
        }

        auto s = tostringview(L);
        lua_pop(L, 1);
        auto f =  ::emilua::gperf::detail::value_or(::emilua::gperf::detail::Perfect_Hash_3::in_word_set((s).data(), (s).size()), {});
        if (f == 0) {
            push(L, std::errc::invalid_argument, "arg", 2);
            return lua_error(L);
        }
        flags |= f;
    }
 end_for:

    asio_error_code ec;
    (*ctx)->set_options(flags, ec);
    if (ec) {
        push(L, ec);
        return lua_error(L);
    }
    return 0;
}

static int context_set_password_callback(lua_State* L)
{
    luaL_checktype(L, 2, LUA_TFUNCTION);

    auto& vm_ctx = get_vm_context(L);

    auto ctx = static_cast<std::shared_ptr<asio::ssl::context>*>(
        lua_touserdata(L, 1)
    );
    if (!ctx || !lua_getmetatable(L, 1)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }
    rawgetp(L, LUA_REGISTRYINDEX, &tls_context_mt_key);
    if (!lua_rawequal(L, -1, -2)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }

    lua_pushvalue(L, 2);
    int ref = luaL_ref(L, LUA_REGISTRYINDEX);

    asio_error_code ec;
    (*ctx)->set_password_callback(context_password_callback{vm_ctx, ref}, ec);
    if (ec) {
        push(L, static_cast<std::error_code>(ec));
        return lua_error(L);
    }
    return 0;
}

static int context_set_verify_callback(lua_State* L)
{
    lua_settop(L, 3);
    luaL_checktype(L, 2, LUA_TSTRING);

    auto ctx = static_cast<std::shared_ptr<asio::ssl::context>*>(
        lua_touserdata(L, 1)
    );
    if (!ctx || !lua_getmetatable(L, 1)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }
    rawgetp(L, LUA_REGISTRYINDEX, &tls_context_mt_key);
    if (!lua_rawequal(L, -1, -2)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }

    auto key = tostringview(L, 2);
    return  ::emilua::gperf::detail::value_or(::emilua::gperf::detail::Perfect_Hash_4::in_word_set((key).data(), (key).size()), 
            [](lua_State* L, std::shared_ptr<asio::ssl::context>*) -> int {
                push(L, std::errc::not_supported);
                return lua_error(L);
            })(L, ctx);
}

static int context_set_verify_depth(lua_State* L)
{
    luaL_checktype(L, 2, LUA_TNUMBER);

    auto ctx = static_cast<std::shared_ptr<asio::ssl::context>*>(
        lua_touserdata(L, 1)
    );
    if (!ctx || !lua_getmetatable(L, 1)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }
    rawgetp(L, LUA_REGISTRYINDEX, &tls_context_mt_key);
    if (!lua_rawequal(L, -1, -2)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }

    asio_error_code ec;
    (*ctx)->set_verify_depth(lua_tointeger(L, 2), ec);
    if (ec) {
        push(L, ec);
        return lua_error(L);
    }
    return 0;
}

static int context_set_verify_mode(lua_State* L)
{
    luaL_checktype(L, 2, LUA_TSTRING);

    auto ctx = static_cast<std::shared_ptr<asio::ssl::context>*>(
        lua_touserdata(L, 1)
    );
    if (!ctx || !lua_getmetatable(L, 1)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }
    rawgetp(L, LUA_REGISTRYINDEX, &tls_context_mt_key);
    if (!lua_rawequal(L, -1, -2)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }

    auto key = tostringview(L, 2);
    return  ::emilua::gperf::detail::value_or(::emilua::gperf::detail::Perfect_Hash_5::in_word_set((key).data(), (key).size()), 
            [](lua_State* L, std::shared_ptr<asio::ssl::context>*) -> int {
                push(L, std::errc::invalid_argument, "arg", 2);
                return lua_error(L);
            })(L, ctx);
}

static int context_use_certificate(lua_State* L)
{
    luaL_checktype(L, 3, LUA_TSTRING);

    auto ctx = static_cast<std::shared_ptr<asio::ssl::context>*>(
        lua_touserdata(L, 1)
    );
    if (!ctx || !lua_getmetatable(L, 1)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }
    rawgetp(L, LUA_REGISTRYINDEX, &tls_context_mt_key);
    if (!lua_rawequal(L, -1, -2)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }

    auto bs = static_cast<byte_span_handle*>(lua_touserdata(L, 2));
    if (!bs || !lua_getmetatable(L, 2)) {
        push(L, std::errc::invalid_argument, "arg", 2);
        return lua_error(L);
    }
    rawgetp(L, LUA_REGISTRYINDEX, &byte_span_mt_key);
    if (!lua_rawequal(L, -1, -2)) {
        push(L, std::errc::invalid_argument, "arg", 2);
        return lua_error(L);
    }

    asio::ssl::context::file_format fmt;
    auto fmt_str = tostringview(L, 3);
    if (fmt_str == "pem") {
        fmt = asio::ssl::context::pem;
    } else if (fmt_str == "asn1") {
        fmt = asio::ssl::context::asn1;
    } else {
        push(L, std::errc::invalid_argument, "arg", 3);
        return lua_error(L);
    }

    asio_error_code ec;
    (*ctx)->use_certificate(asio::buffer(bs->data.get(), bs->size), fmt, ec);
    if (ec) {
        push(L, ec);
        return lua_error(L);
    }
    return 0;
}

static int context_use_certificate_chain(lua_State* L)
{
    auto ctx = static_cast<std::shared_ptr<asio::ssl::context>*>(
        lua_touserdata(L, 1)
    );
    if (!ctx || !lua_getmetatable(L, 1)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }
    rawgetp(L, LUA_REGISTRYINDEX, &tls_context_mt_key);
    if (!lua_rawequal(L, -1, -2)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }

    auto bs = static_cast<byte_span_handle*>(lua_touserdata(L, 2));
    if (!bs || !lua_getmetatable(L, 2)) {
        push(L, std::errc::invalid_argument, "arg", 2);
        return lua_error(L);
    }
    rawgetp(L, LUA_REGISTRYINDEX, &byte_span_mt_key);
    if (!lua_rawequal(L, -1, -2)) {
        push(L, std::errc::invalid_argument, "arg", 2);
        return lua_error(L);
    }

    asio_error_code ec;
    (*ctx)->use_certificate_chain(asio::buffer(bs->data.get(), bs->size), ec);
    if (ec) {
        push(L, ec);
        return lua_error(L);
    }
    return 0;
}

static int context_use_certificate_chain_file(lua_State* L)
{
    lua_settop(L, 2);

    auto ctx = static_cast<std::shared_ptr<asio::ssl::context>*>(
        lua_touserdata(L, 1)
    );
    if (!ctx || !lua_getmetatable(L, 1)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }
    rawgetp(L, LUA_REGISTRYINDEX, &tls_context_mt_key);
    if (!lua_rawequal(L, -1, -2)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }

    std::string path;
    try {
        auto p = static_cast<std::filesystem::path*>(lua_touserdata(L, 2));
        if (!p || !lua_getmetatable(L, 2)) {
            push(L, std::errc::invalid_argument, "arg", 2);
            return lua_error(L);
        }
        rawgetp(L, LUA_REGISTRYINDEX, &filesystem_path_mt_key);
        if (!lua_rawequal(L, -1, -2)) {
            push(L, std::errc::invalid_argument, "arg", 2);
            return lua_error(L);
        }

        path = p->string();
    } catch (const std::system_error& e) {
        push(L, e.code());
        return lua_error(L);
    } catch (const std::exception& e) {
        lua_pushstring(L, e.what());
        return lua_error(L);
    }

    asio_error_code ec;
    (*ctx)->use_certificate_chain_file(path, ec);
    if (ec) {
        push(L, ec);
        return lua_error(L);
    }
    return 0;
}

static int context_use_certificate_file(lua_State* L)
{
    luaL_checktype(L, 3, LUA_TSTRING);

    auto ctx = static_cast<std::shared_ptr<asio::ssl::context>*>(
        lua_touserdata(L, 1)
    );
    if (!ctx || !lua_getmetatable(L, 1)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }
    rawgetp(L, LUA_REGISTRYINDEX, &tls_context_mt_key);
    if (!lua_rawequal(L, -1, -2)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }

    std::string path;
    try {
        auto p = static_cast<std::filesystem::path*>(lua_touserdata(L, 2));
        if (!p || !lua_getmetatable(L, 2)) {
            push(L, std::errc::invalid_argument, "arg", 2);
            return lua_error(L);
        }
        rawgetp(L, LUA_REGISTRYINDEX, &filesystem_path_mt_key);
        if (!lua_rawequal(L, -1, -2)) {
            push(L, std::errc::invalid_argument, "arg", 2);
            return lua_error(L);
        }

        path = p->string();
    } catch (const std::system_error& e) {
        push(L, e.code());
        return lua_error(L);
    } catch (const std::exception& e) {
        lua_pushstring(L, e.what());
        return lua_error(L);
    }

    asio::ssl::context::file_format fmt;
    auto fmt_str = tostringview(L, 3);
    if (fmt_str == "pem") {
        fmt = asio::ssl::context::pem;
    } else if (fmt_str == "asn1") {
        fmt = asio::ssl::context::asn1;
    } else {
        push(L, std::errc::invalid_argument, "arg", 3);
        return lua_error(L);
    }

    asio_error_code ec;
    (*ctx)->use_certificate_file(path, fmt, ec);
    if (ec) {
        push(L, ec);
        return lua_error(L);
    }
    return 0;
}

static int context_use_private_key(lua_State* L)
{
    luaL_checktype(L, 3, LUA_TSTRING);

    auto ctx = static_cast<std::shared_ptr<asio::ssl::context>*>(
        lua_touserdata(L, 1)
    );
    if (!ctx || !lua_getmetatable(L, 1)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }
    rawgetp(L, LUA_REGISTRYINDEX, &tls_context_mt_key);
    if (!lua_rawequal(L, -1, -2)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }

    auto bs = static_cast<byte_span_handle*>(lua_touserdata(L, 2));
    if (!bs || !lua_getmetatable(L, 2)) {
        push(L, std::errc::invalid_argument, "arg", 2);
        return lua_error(L);
    }
    rawgetp(L, LUA_REGISTRYINDEX, &byte_span_mt_key);
    if (!lua_rawequal(L, -1, -2)) {
        push(L, std::errc::invalid_argument, "arg", 2);
        return lua_error(L);
    }

    asio::ssl::context::file_format fmt;
    auto fmt_str = tostringview(L, 3);
    if (fmt_str == "pem") {
        fmt = asio::ssl::context::pem;
    } else if (fmt_str == "asn1") {
        fmt = asio::ssl::context::asn1;
    } else {
        push(L, std::errc::invalid_argument, "arg", 3);
        return lua_error(L);
    }

    asio_error_code ec;
    (*ctx)->use_private_key(asio::buffer(bs->data.get(), bs->size), fmt, ec);
    if (ec) {
        push(L, ec);
        return lua_error(L);
    }
    return 0;
}

static int context_use_private_key_file(lua_State* L)
{
    luaL_checktype(L, 3, LUA_TSTRING);

    auto ctx = static_cast<std::shared_ptr<asio::ssl::context>*>(
        lua_touserdata(L, 1)
    );
    if (!ctx || !lua_getmetatable(L, 1)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }
    rawgetp(L, LUA_REGISTRYINDEX, &tls_context_mt_key);
    if (!lua_rawequal(L, -1, -2)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }

    std::string path;
    try {
        auto p = static_cast<std::filesystem::path*>(lua_touserdata(L, 2));
        if (!p || !lua_getmetatable(L, 2)) {
            push(L, std::errc::invalid_argument, "arg", 2);
            return lua_error(L);
        }
        rawgetp(L, LUA_REGISTRYINDEX, &filesystem_path_mt_key);
        if (!lua_rawequal(L, -1, -2)) {
            push(L, std::errc::invalid_argument, "arg", 2);
            return lua_error(L);
        }

        path = p->string();
    } catch (const std::system_error& e) {
        push(L, e.code());
        return lua_error(L);
    } catch (const std::exception& e) {
        lua_pushstring(L, e.what());
        return lua_error(L);
    }

    asio::ssl::context::file_format fmt;
    auto fmt_str = tostringview(L, 3);
    if (fmt_str == "pem") {
        fmt = asio::ssl::context::pem;
    } else if (fmt_str == "asn1") {
        fmt = asio::ssl::context::asn1;
    } else {
        push(L, std::errc::invalid_argument, "arg", 3);
        return lua_error(L);
    }

    asio_error_code ec;
    (*ctx)->use_private_key_file(path, fmt, ec);
    if (ec) {
        push(L, ec);
        return lua_error(L);
    }
    return 0;
}

static int context_use_rsa_private_key(lua_State* L)
{
    luaL_checktype(L, 3, LUA_TSTRING);

    auto ctx = static_cast<std::shared_ptr<asio::ssl::context>*>(
        lua_touserdata(L, 1)
    );
    if (!ctx || !lua_getmetatable(L, 1)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }
    rawgetp(L, LUA_REGISTRYINDEX, &tls_context_mt_key);
    if (!lua_rawequal(L, -1, -2)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }

    auto bs = static_cast<byte_span_handle*>(lua_touserdata(L, 2));
    if (!bs || !lua_getmetatable(L, 2)) {
        push(L, std::errc::invalid_argument, "arg", 2);
        return lua_error(L);
    }
    rawgetp(L, LUA_REGISTRYINDEX, &byte_span_mt_key);
    if (!lua_rawequal(L, -1, -2)) {
        push(L, std::errc::invalid_argument, "arg", 2);
        return lua_error(L);
    }

    asio::ssl::context::file_format fmt;
    auto fmt_str = tostringview(L, 3);
    if (fmt_str == "pem") {
        fmt = asio::ssl::context::pem;
    } else if (fmt_str == "asn1") {
        fmt = asio::ssl::context::asn1;
    } else {
        push(L, std::errc::invalid_argument, "arg", 3);
        return lua_error(L);
    }

    asio_error_code ec;
    (*ctx)->use_rsa_private_key(
        asio::buffer(bs->data.get(), bs->size), fmt, ec);
    if (ec) {
        push(L, ec);
        return lua_error(L);
    }
    return 0;
}

static int context_use_rsa_private_key_file(lua_State* L)
{
    luaL_checktype(L, 3, LUA_TSTRING);

    auto ctx = static_cast<std::shared_ptr<asio::ssl::context>*>(
        lua_touserdata(L, 1)
    );
    if (!ctx || !lua_getmetatable(L, 1)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }
    rawgetp(L, LUA_REGISTRYINDEX, &tls_context_mt_key);
    if (!lua_rawequal(L, -1, -2)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }

    std::string path;
    try {
        auto p = static_cast<std::filesystem::path*>(lua_touserdata(L, 2));
        if (!p || !lua_getmetatable(L, 2)) {
            push(L, std::errc::invalid_argument, "arg", 2);
            return lua_error(L);
        }
        rawgetp(L, LUA_REGISTRYINDEX, &filesystem_path_mt_key);
        if (!lua_rawequal(L, -1, -2)) {
            push(L, std::errc::invalid_argument, "arg", 2);
            return lua_error(L);
        }

        path = p->string();
    } catch (const std::system_error& e) {
        push(L, e.code());
        return lua_error(L);
    } catch (const std::exception& e) {
        lua_pushstring(L, e.what());
        return lua_error(L);
    }

    asio::ssl::context::file_format fmt;
    auto fmt_str = tostringview(L, 3);
    if (fmt_str == "pem") {
        fmt = asio::ssl::context::pem;
    } else if (fmt_str == "asn1") {
        fmt = asio::ssl::context::asn1;
    } else {
        push(L, std::errc::invalid_argument, "arg", 3);
        return lua_error(L);
    }

    asio_error_code ec;
    (*ctx)->use_rsa_private_key_file(path, fmt, ec);
    if (ec) {
        push(L, ec);
        return lua_error(L);
    }
    return 0;
}

static int context_use_tmp_dh(lua_State* L)
{
    auto ctx = static_cast<std::shared_ptr<asio::ssl::context>*>(
        lua_touserdata(L, 1)
    );
    if (!ctx || !lua_getmetatable(L, 1)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }
    rawgetp(L, LUA_REGISTRYINDEX, &tls_context_mt_key);
    if (!lua_rawequal(L, -1, -2)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }

    auto bs = static_cast<byte_span_handle*>(lua_touserdata(L, 2));
    if (!bs || !lua_getmetatable(L, 2)) {
        push(L, std::errc::invalid_argument, "arg", 2);
        return lua_error(L);
    }
    rawgetp(L, LUA_REGISTRYINDEX, &byte_span_mt_key);
    if (!lua_rawequal(L, -1, -2)) {
        push(L, std::errc::invalid_argument, "arg", 2);
        return lua_error(L);
    }

    asio_error_code ec;
    (*ctx)->use_tmp_dh(asio::buffer(bs->data.get(), bs->size), ec);
    if (ec) {
        push(L, ec);
        return lua_error(L);
    }
    return 0;
}

static int context_use_tmp_dh_file(lua_State* L)
{
    lua_settop(L, 2);

    auto ctx = static_cast<std::shared_ptr<asio::ssl::context>*>(
        lua_touserdata(L, 1)
    );
    if (!ctx || !lua_getmetatable(L, 1)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }
    rawgetp(L, LUA_REGISTRYINDEX, &tls_context_mt_key);
    if (!lua_rawequal(L, -1, -2)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }

    std::string path;
    try {
        auto p = static_cast<std::filesystem::path*>(lua_touserdata(L, 2));
        if (!p || !lua_getmetatable(L, 2)) {
            push(L, std::errc::invalid_argument, "arg", 2);
            return lua_error(L);
        }
        rawgetp(L, LUA_REGISTRYINDEX, &filesystem_path_mt_key);
        if (!lua_rawequal(L, -1, -2)) {
            push(L, std::errc::invalid_argument, "arg", 2);
            return lua_error(L);
        }

        path = p->string();
    } catch (const std::system_error& e) {
        push(L, e.code());
        return lua_error(L);
    } catch (const std::exception& e) {
        lua_pushstring(L, e.what());
        return lua_error(L);
    }

    asio_error_code ec;
    (*ctx)->use_tmp_dh_file(path, ec);
    if (ec) {
        push(L, ec);
        return lua_error(L);
    }
    return 0;
}

} // namespace emilua

#line 1644 "libemilua.so.0.11.5.p/bc2_tls.cpp"
namespace emilua::gperf::detail {
namespace {
/* C++ code produced by gperf version 3.1 */
/* Command-line: /usr/bin/gperf --language=C++ --enum --readonly-tables --struct-type --initializer-suffix=',{}' --class-name=Perfect_Hash_6  */
/* Computed positions: -k'12' */

#if !((' ' == 32) && ('!' == 33) && ('"' == 34) && ('#' == 35) \
      && ('%' == 37) && ('&' == 38) && ('\'' == 39) && ('(' == 40) \
      && (')' == 41) && ('*' == 42) && ('+' == 43) && (',' == 44) \
      && ('-' == 45) && ('.' == 46) && ('/' == 47) && ('0' == 48) \
      && ('1' == 49) && ('2' == 50) && ('3' == 51) && ('4' == 52) \
      && ('5' == 53) && ('6' == 54) && ('7' == 55) && ('8' == 56) \
      && ('9' == 57) && (':' == 58) && (';' == 59) && ('<' == 60) \
      && ('=' == 61) && ('>' == 62) && ('?' == 63) && ('A' == 65) \
      && ('B' == 66) && ('C' == 67) && ('D' == 68) && ('E' == 69) \
      && ('F' == 70) && ('G' == 71) && ('H' == 72) && ('I' == 73) \
      && ('J' == 74) && ('K' == 75) && ('L' == 76) && ('M' == 77) \
      && ('N' == 78) && ('O' == 79) && ('P' == 80) && ('Q' == 81) \
      && ('R' == 82) && ('S' == 83) && ('T' == 84) && ('U' == 85) \
      && ('V' == 86) && ('W' == 87) && ('X' == 88) && ('Y' == 89) \
      && ('Z' == 90) && ('[' == 91) && ('\\' == 92) && (']' == 93) \
      && ('^' == 94) && ('_' == 95) && ('a' == 97) && ('b' == 98) \
      && ('c' == 99) && ('d' == 100) && ('e' == 101) && ('f' == 102) \
      && ('g' == 103) && ('h' == 104) && ('i' == 105) && ('j' == 106) \
      && ('k' == 107) && ('l' == 108) && ('m' == 109) && ('n' == 110) \
      && ('o' == 111) && ('p' == 112) && ('q' == 113) && ('r' == 114) \
      && ('s' == 115) && ('t' == 116) && ('u' == 117) && ('v' == 118) \
      && ('w' == 119) && ('x' == 120) && ('y' == 121) && ('z' == 122) \
      && ('{' == 123) && ('|' == 124) && ('}' == 125) && ('~' == 126))
/* The character set is not based on ISO-646.  */
#error "gperf generated tables don't work with this execution character set. Please report a bug to <bug-gperf@gnu.org>."
#endif

struct word_type_6 { const char* name; int (*action)(lua_State*); };
/* maximum key range = 36, duplicates = 0 */

class Perfect_Hash_6
{
private:
  static inline unsigned int hash (const char *str, size_t len);
public:
  static const struct word_type_6 *in_word_set (const char *str, size_t len);
};

inline unsigned int
Perfect_Hash_6::hash (const char *str, size_t len)
{
  static const unsigned char asso_values[] =
    {
      46, 46, 46, 46, 46, 46, 46, 46, 46, 46,
      46, 46, 46, 46, 46, 46, 46, 46, 46, 46,
      46, 46, 46, 46, 46, 46, 46, 46, 46, 46,
      46, 46, 46, 46, 46, 46, 46, 46, 46, 46,
      46, 46, 46, 46, 46, 46, 46, 46, 46, 46,
      46, 46, 46, 46, 46, 46, 46, 46, 46, 46,
      46, 46, 46, 46, 46, 46, 46, 46, 46, 46,
      46, 46, 46, 46, 46, 46, 46, 46, 46, 46,
      46, 46, 46, 46, 46, 46, 46, 46, 46, 46,
      46, 46, 46, 46, 46, 15, 46, 46, 46,  0,
      20, 46,  3, 46, 46, 46, 46, 46, 46, 30,
       0, 46, 25, 46, 46, 46, 46, 46,  5, 46,
      46, 46, 46, 46, 46, 46, 46, 46, 46, 46,
      46, 46, 46, 46, 46, 46, 46, 46, 46, 46,
      46, 46, 46, 46, 46, 46, 46, 46, 46, 46,
      46, 46, 46, 46, 46, 46, 46, 46, 46, 46,
      46, 46, 46, 46, 46, 46, 46, 46, 46, 46,
      46, 46, 46, 46, 46, 46, 46, 46, 46, 46,
      46, 46, 46, 46, 46, 46, 46, 46, 46, 46,
      46, 46, 46, 46, 46, 46, 46, 46, 46, 46,
      46, 46, 46, 46, 46, 46, 46, 46, 46, 46,
      46, 46, 46, 46, 46, 46, 46, 46, 46, 46,
      46, 46, 46, 46, 46, 46, 46, 46, 46, 46,
      46, 46, 46, 46, 46, 46, 46, 46, 46, 46,
      46, 46, 46, 46, 46, 46, 46, 46, 46, 46,
      46, 46, 46, 46, 46, 46
    };
  unsigned int hval = len;

  switch (hval)
    {
      default:
        hval += asso_values[static_cast<unsigned char>(str[11])];
      /*FALLTHROUGH*/
      case 11:
      case 10:
        break;
    }
  return hval;
}

const struct word_type_6 *
Perfect_Hash_6::in_word_set (const char *str, size_t len)
{
  enum
    {
      TOTAL_KEYWORDS = 20,
      MIN_WORD_LENGTH = 10,
      MAX_WORD_LENGTH = 26,
      MIN_HASH_VALUE = 10,
      MAX_HASH_VALUE = 45
    };

  static const struct word_type_6 wordlist[] =
    {
      {"",{}}, {"",{}}, {"",{}}, {"",{}}, {"",{}}, {"",{}},
      {"",{}}, {"",{}}, {"",{}}, {"",{}},
      {"use_tmp_dh", 
            [](lua_State* L) -> int {
                lua_pushcfunction(L, context_use_tmp_dh);
                return 1;
            }},
      {"set_options", 
            [](lua_State* L) -> int {
                lua_pushcfunction(L, context_set_options);
                return 1;
            }},
      {"",{}},
      {"clear_options", 
            [](lua_State* L) -> int {
                lua_pushcfunction(L, context_clear_options);
                return 1;
            }},
      {"",{}},
      {"use_certificate", 
            [](lua_State* L) -> int {
                lua_pushcfunction(L, context_use_certificate);
                return 1;
            }},
      {"",{}}, {"",{}},
      {"use_tmp_dh_file", 
            [](lua_State* L) -> int {
                lua_pushcfunction(L, context_use_tmp_dh_file);
                return 1;
            }},
      {"set_verify_callback", 
            [](lua_State* L) -> int {
                lua_pushcfunction(L, context_set_verify_callback);
                return 1;
            }},
      {"use_certificate_file", 
            [](lua_State* L) -> int {
                lua_pushcfunction(L, context_use_certificate_file);
                return 1;
            }},
      {"use_certificate_chain", 
            [](lua_State* L) -> int {
                lua_pushcfunction(L, context_use_certificate_chain);
                return 1;
            }},
      {"",{}}, {"",{}},
      {"use_rsa_private_key", 
            [](lua_State* L) -> int {
                lua_pushcfunction(L, context_use_rsa_private_key);
                return 1;
            }},
      {"add_certificate_authority", 
            [](lua_State* L) -> int {
                lua_pushcfunction(L, context_add_certificate_authority);
                return 1;
            }},
      {"use_certificate_chain_file", 
            [](lua_State* L) -> int {
                lua_pushcfunction(L, context_use_certificate_chain_file);
                return 1;
            }},
      {"",{}}, {"",{}},
      {"use_rsa_private_key_file", 
            [](lua_State* L) -> int {
                lua_pushcfunction(L, context_use_rsa_private_key_file);
                return 1;
            }},
      {"use_private_key", 
            [](lua_State* L) -> int {
                lua_pushcfunction(L, context_use_private_key);
                return 1;
            }},
      {"load_verify_file", 
            [](lua_State* L) -> int {
                lua_pushcfunction(L, context_load_verify_file);
                return 1;
            }},
      {"",{}}, {"",{}}, {"",{}},
      {"use_private_key_file", 
            [](lua_State* L) -> int {
                lua_pushcfunction(L, context_use_private_key_file);
                return 1;
            }},
      {"set_verify_depth", 
            [](lua_State* L) -> int {
                lua_pushcfunction(L, context_set_verify_depth);
                return 1;
            }},
      {"",{}}, {"",{}},
      {"set_default_verify_paths", 
            [](lua_State* L) -> int {
                lua_pushcfunction(L, context_set_default_verify_paths);
                return 1;
            }},
      {"add_verify_path", 
            [](lua_State* L) -> int {
                lua_pushcfunction(L, context_add_verify_path);
                return 1;
            }},
      {"set_password_callback", 
            [](lua_State* L) -> int {
                lua_pushcfunction(L, context_set_password_callback);
                return 1;
            }},
      {"",{}}, {"",{}}, {"",{}},
      {"set_verify_mode", 
            [](lua_State* L) -> int {
                lua_pushcfunction(L, context_set_verify_mode);
                return 1;
            }}
    };

  if (len <= MAX_WORD_LENGTH && len >= MIN_WORD_LENGTH)
    {
      unsigned int key = hash (str, len);

      if (key <= MAX_HASH_VALUE)
        {
          const char *s = wordlist[key].name;

          if (*str == *s && !strcmp (str + 1, s + 1))
            return &wordlist[key];
        }
    }
  return 0;
}

} // namespace
} // namespace emilua::gperf::detail

#line 1879 "libemilua.so.0.11.5.p/bc2_tls.cpp"
namespace emilua::gperf::detail {
namespace {
/* C++ code produced by gperf version 3.1 */
/* Command-line: /usr/bin/gperf --language=C++ --enum --readonly-tables --struct-type --initializer-suffix=',{}' --class-name=Perfect_Hash_7  */
/* Computed positions: -k'' */

struct word_type_7 { const char* name; int (*action)(lua_State*, TlsSocket*); };
/* maximum key range = 1, duplicates = 0 */

class Perfect_Hash_7
{
private:
  static inline unsigned int hash (const char *str, size_t len);
public:
  static const struct word_type_7 *in_word_set (const char *str, size_t len);
};

inline /*ARGSUSED*/
unsigned int
Perfect_Hash_7::hash (const char *str, size_t len)
{
  return 0;
}

const struct word_type_7 *
Perfect_Hash_7::in_word_set (const char *str, size_t len)
{
  enum
    {
      TOTAL_KEYWORDS = 1,
      MIN_WORD_LENGTH = 22,
      MAX_WORD_LENGTH = 22,
      MIN_HASH_VALUE = 0,
      MAX_HASH_VALUE = 0
    };

  static const struct word_type_7 wordlist[] =
    {
      {"host_name_verification", 
            [](lua_State* L, TlsSocket* s) -> int {
                luaL_checktype(L, 3, LUA_TSTRING);
                asio::ssl::host_name_verification o{
                    static_cast<std::string>(tostringview(L, 3))};
                asio_error_code ec;
                s->set_verify_callback(std::move(o), ec);
                if (ec) {
                    push(L, static_cast<std::error_code>(ec));
                    return lua_error(L);
                }
                return 0;
            }}
    };

  if (len <= MAX_WORD_LENGTH && len >= MIN_WORD_LENGTH)
    {
      unsigned int key = hash (str, len);

      if (key <= MAX_HASH_VALUE)
        {
          const char *s = wordlist[key].name;

          if (*str == *s && !strcmp (str + 1, s + 1))
            return &wordlist[key];
        }
    }
  return 0;
}

} // namespace
} // namespace emilua::gperf::detail

#line 1951 "libemilua.so.0.11.5.p/bc2_tls.cpp"
namespace emilua::gperf::detail {
namespace {
/* C++ code produced by gperf version 3.1 */
/* Command-line: /usr/bin/gperf --language=C++ --enum --readonly-tables --struct-type --initializer-suffix=',{}' --class-name=Perfect_Hash_8  */
/* Computed positions: -k'1' */

#if !((' ' == 32) && ('!' == 33) && ('"' == 34) && ('#' == 35) \
      && ('%' == 37) && ('&' == 38) && ('\'' == 39) && ('(' == 40) \
      && (')' == 41) && ('*' == 42) && ('+' == 43) && (',' == 44) \
      && ('-' == 45) && ('.' == 46) && ('/' == 47) && ('0' == 48) \
      && ('1' == 49) && ('2' == 50) && ('3' == 51) && ('4' == 52) \
      && ('5' == 53) && ('6' == 54) && ('7' == 55) && ('8' == 56) \
      && ('9' == 57) && (':' == 58) && (';' == 59) && ('<' == 60) \
      && ('=' == 61) && ('>' == 62) && ('?' == 63) && ('A' == 65) \
      && ('B' == 66) && ('C' == 67) && ('D' == 68) && ('E' == 69) \
      && ('F' == 70) && ('G' == 71) && ('H' == 72) && ('I' == 73) \
      && ('J' == 74) && ('K' == 75) && ('L' == 76) && ('M' == 77) \
      && ('N' == 78) && ('O' == 79) && ('P' == 80) && ('Q' == 81) \
      && ('R' == 82) && ('S' == 83) && ('T' == 84) && ('U' == 85) \
      && ('V' == 86) && ('W' == 87) && ('X' == 88) && ('Y' == 89) \
      && ('Z' == 90) && ('[' == 91) && ('\\' == 92) && (']' == 93) \
      && ('^' == 94) && ('_' == 95) && ('a' == 97) && ('b' == 98) \
      && ('c' == 99) && ('d' == 100) && ('e' == 101) && ('f' == 102) \
      && ('g' == 103) && ('h' == 104) && ('i' == 105) && ('j' == 106) \
      && ('k' == 107) && ('l' == 108) && ('m' == 109) && ('n' == 110) \
      && ('o' == 111) && ('p' == 112) && ('q' == 113) && ('r' == 114) \
      && ('s' == 115) && ('t' == 116) && ('u' == 117) && ('v' == 118) \
      && ('w' == 119) && ('x' == 120) && ('y' == 121) && ('z' == 122) \
      && ('{' == 123) && ('|' == 124) && ('}' == 125) && ('~' == 126))
/* The character set is not based on ISO-646.  */
#error "gperf generated tables don't work with this execution character set. Please report a bug to <bug-gperf@gnu.org>."
#endif

struct word_type_8 { const char* name; int (*action)(lua_State*, TlsSocket*); };
/* maximum key range = 17, duplicates = 0 */

class Perfect_Hash_8
{
private:
  static inline unsigned int hash (const char *str, size_t len);
public:
  static const struct word_type_8 *in_word_set (const char *str, size_t len);
};

inline unsigned int
Perfect_Hash_8::hash (const char *str, size_t len)
{
  static const unsigned char asso_values[] =
    {
      21, 21, 21, 21, 21, 21, 21, 21, 21, 21,
      21, 21, 21, 21, 21, 21, 21, 21, 21, 21,
      21, 21, 21, 21, 21, 21, 21, 21, 21, 21,
      21, 21, 21, 21, 21, 21, 21, 21, 21, 21,
      21, 21, 21, 21, 21, 21, 21, 21, 21, 21,
      21, 21, 21, 21, 21, 21, 21, 21, 21, 21,
      21, 21, 21, 21, 21, 21, 21, 21, 21, 21,
      21, 21, 21, 21, 21, 21, 21, 21, 21, 21,
      21, 21, 21, 21, 21, 21, 21, 21, 21, 21,
      21, 21, 21, 21, 21, 21, 21, 21, 21,  0,
      21, 21,  0, 21, 21, 21, 21, 21, 21, 21,
       5, 21,  0, 21, 21, 21, 21, 21, 21, 21,
      21, 21, 21, 21, 21, 21, 21, 21, 21, 21,
      21, 21, 21, 21, 21, 21, 21, 21, 21, 21,
      21, 21, 21, 21, 21, 21, 21, 21, 21, 21,
      21, 21, 21, 21, 21, 21, 21, 21, 21, 21,
      21, 21, 21, 21, 21, 21, 21, 21, 21, 21,
      21, 21, 21, 21, 21, 21, 21, 21, 21, 21,
      21, 21, 21, 21, 21, 21, 21, 21, 21, 21,
      21, 21, 21, 21, 21, 21, 21, 21, 21, 21,
      21, 21, 21, 21, 21, 21, 21, 21, 21, 21,
      21, 21, 21, 21, 21, 21, 21, 21, 21, 21,
      21, 21, 21, 21, 21, 21, 21, 21, 21, 21,
      21, 21, 21, 21, 21, 21, 21, 21, 21, 21,
      21, 21, 21, 21, 21, 21, 21, 21, 21, 21,
      21, 21, 21, 21, 21, 21
    };
  return len + asso_values[static_cast<unsigned char>(str[0])];
}

const struct word_type_8 *
Perfect_Hash_8::in_word_set (const char *str, size_t len)
{
  enum
    {
      TOTAL_KEYWORDS = 4,
      MIN_WORD_LENGTH = 4,
      MAX_WORD_LENGTH = 20,
      MIN_HASH_VALUE = 4,
      MAX_HASH_VALUE = 20
    };

  static const struct word_type_8 wordlist[] =
    {
      {"",{}}, {"",{}}, {"",{}}, {"",{}},
      {"peer", 
            [](lua_State* L, TlsSocket* s) -> int {
                asio_error_code ec;
                s->set_verify_mode(asio::ssl::verify_peer, ec);
                if (ec) {
                    push(L, static_cast<std::error_code>(ec));
                    return lua_error(L);
                }
                return 0;
            }},
      {"",{}}, {"",{}}, {"",{}}, {"",{}},
      {"none", 
            [](lua_State* L, TlsSocket* s) -> int {
                asio_error_code ec;
                s->set_verify_mode(asio::ssl::verify_none, ec);
                if (ec) {
                    push(L, static_cast<std::error_code>(ec));
                    return lua_error(L);
                }
                return 0;
            }},
      {"",{}},
      {"client_once", 
            [](lua_State* L, TlsSocket* s) -> int {
                asio_error_code ec;
                s->set_verify_mode(asio::ssl::verify_client_once, ec);
                if (ec) {
                    push(L, static_cast<std::error_code>(ec));
                    return lua_error(L);
                }
                return 0;
            }},
      {"",{}}, {"",{}}, {"",{}}, {"",{}}, {"",{}}, {"",{}},
      {"",{}}, {"",{}},
      {"fail_if_no_peer_cert", 
            [](lua_State* L, TlsSocket* s) -> int {
                asio_error_code ec;
                s->set_verify_mode(asio::ssl::verify_fail_if_no_peer_cert, ec);
                if (ec) {
                    push(L, static_cast<std::error_code>(ec));
                    return lua_error(L);
                }
                return 0;
            }}
    };

  if (len <= MAX_WORD_LENGTH && len >= MIN_WORD_LENGTH)
    {
      unsigned int key = hash (str, len);

      if (key <= MAX_HASH_VALUE)
        {
          const char *s = wordlist[key].name;

          if (*str == *s && !strcmp (str + 1, s + 1))
            return &wordlist[key];
        }
    }
  return 0;
}

} // namespace
} // namespace emilua::gperf::detail

#line 2110 "libemilua.so.0.11.5.p/bc2_tls.cpp"
namespace emilua {


#if (EMILUA_CONFIG_USE_STANDALONE_ASIO && !defined(ASIO_USE_WOLFSSL)) || (!EMILUA_CONFIG_USE_STANDALONE_ASIO && !defined(BOOST_ASIO_USE_WOLFSSL))
static int tls_socket_set_server_name(lua_State* L)
{
    luaL_checktype(L, 2, LUA_TSTRING);

    auto s = static_cast<TlsSocket*>(lua_touserdata(L, 1));
    if (!s || !lua_getmetatable(L, 1)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }
    rawgetp(L, LUA_REGISTRYINDEX, &tls_socket_mt_key);
    if (!lua_rawequal(L, -1, -2)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }

    if(!SSL_set_tlsext_host_name(s->native_handle(), lua_tostring(L, 2))) {
        asio_error_code ec{
            static_cast<int>(ERR_get_error()), asio::error::get_ssl_category()
        };
        push(L, ec);
        return lua_error(L);
    }

    return 0;
}
#endif // (EMILUA_CONFIG_USE_STANDALONE_ASIO && !defined(ASIO_USE_WOLFSSL)) || (!EMILUA_CONFIG_USE_STANDALONE_ASIO && !defined(BOOST_ASIO_USE_WOLFSSL))

static int tls_socket_set_verify_callback(lua_State* L)
{
    lua_settop(L, 3);
    luaL_checktype(L, 2, LUA_TSTRING);

    auto s = static_cast<TlsSocket*>(lua_touserdata(L, 1));
    if (!s || !lua_getmetatable(L, 1)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }
    rawgetp(L, LUA_REGISTRYINDEX, &tls_socket_mt_key);
    if (!lua_rawequal(L, -1, -2)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }

    auto key = tostringview(L, 2);
    return  ::emilua::gperf::detail::value_or(::emilua::gperf::detail::Perfect_Hash_7::in_word_set((key).data(), (key).size()), [](lua_State* L, TlsSocket*) -> int {
            push(L, std::errc::not_supported);
            return lua_error(L);
        })(L, s);
}

static int tls_socket_set_verify_depth(lua_State* L)
{
    luaL_checktype(L, 2, LUA_TNUMBER);

    auto s = static_cast<TlsSocket*>(lua_touserdata(L, 1));
    if (!s || !lua_getmetatable(L, 1)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }
    rawgetp(L, LUA_REGISTRYINDEX, &tls_socket_mt_key);
    if (!lua_rawequal(L, -1, -2)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }

    asio_error_code ec;
    s->set_verify_depth(lua_tointeger(L, 2), ec);
    if (ec) {
        push(L, ec);
        return lua_error(L);
    }
    return 0;
}

static int tls_socket_set_verify_mode(lua_State* L)
{
    luaL_checktype(L, 2, LUA_TSTRING);

    auto s = static_cast<TlsSocket*>(lua_touserdata(L, 1));
    if (!s || !lua_getmetatable(L, 1)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }
    rawgetp(L, LUA_REGISTRYINDEX, &tls_socket_mt_key);
    if (!lua_rawequal(L, -1, -2)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }

    auto key = tostringview(L, 2);
    return  ::emilua::gperf::detail::value_or(::emilua::gperf::detail::Perfect_Hash_8::in_word_set((key).data(), (key).size()), [](lua_State* L, TlsSocket*) -> int {
            push(L, std::errc::invalid_argument, "arg", 2);
            return lua_error(L);
        })(L, s);
}

} // namespace emilua

#line 2213 "libemilua.so.0.11.5.p/bc2_tls.cpp"
namespace emilua::gperf::detail {
namespace {
/* C++ code produced by gperf version 3.1 */
/* Command-line: /usr/bin/gperf --language=C++ --enum --readonly-tables --struct-type --initializer-suffix=',{}' --class-name=Perfect_Hash_9  */
/* Computed positions: -k'5' */

#if !((' ' == 32) && ('!' == 33) && ('"' == 34) && ('#' == 35) \
      && ('%' == 37) && ('&' == 38) && ('\'' == 39) && ('(' == 40) \
      && (')' == 41) && ('*' == 42) && ('+' == 43) && (',' == 44) \
      && ('-' == 45) && ('.' == 46) && ('/' == 47) && ('0' == 48) \
      && ('1' == 49) && ('2' == 50) && ('3' == 51) && ('4' == 52) \
      && ('5' == 53) && ('6' == 54) && ('7' == 55) && ('8' == 56) \
      && ('9' == 57) && (':' == 58) && (';' == 59) && ('<' == 60) \
      && ('=' == 61) && ('>' == 62) && ('?' == 63) && ('A' == 65) \
      && ('B' == 66) && ('C' == 67) && ('D' == 68) && ('E' == 69) \
      && ('F' == 70) && ('G' == 71) && ('H' == 72) && ('I' == 73) \
      && ('J' == 74) && ('K' == 75) && ('L' == 76) && ('M' == 77) \
      && ('N' == 78) && ('O' == 79) && ('P' == 80) && ('Q' == 81) \
      && ('R' == 82) && ('S' == 83) && ('T' == 84) && ('U' == 85) \
      && ('V' == 86) && ('W' == 87) && ('X' == 88) && ('Y' == 89) \
      && ('Z' == 90) && ('[' == 91) && ('\\' == 92) && (']' == 93) \
      && ('^' == 94) && ('_' == 95) && ('a' == 97) && ('b' == 98) \
      && ('c' == 99) && ('d' == 100) && ('e' == 101) && ('f' == 102) \
      && ('g' == 103) && ('h' == 104) && ('i' == 105) && ('j' == 106) \
      && ('k' == 107) && ('l' == 108) && ('m' == 109) && ('n' == 110) \
      && ('o' == 111) && ('p' == 112) && ('q' == 113) && ('r' == 114) \
      && ('s' == 115) && ('t' == 116) && ('u' == 117) && ('v' == 118) \
      && ('w' == 119) && ('x' == 120) && ('y' == 121) && ('z' == 122) \
      && ('{' == 123) && ('|' == 124) && ('}' == 125) && ('~' == 126))
/* The character set is not based on ISO-646.  */
#error "gperf generated tables don't work with this execution character set. Please report a bug to <bug-gperf@gnu.org>."
#endif

struct word_type_9 { const char* name; int (*action)(lua_State*); };
/* maximum key range = 18, duplicates = 0 */

class Perfect_Hash_9
{
private:
  static inline unsigned int hash (const char *str, size_t len);
public:
  static const struct word_type_9 *in_word_set (const char *str, size_t len);
};

inline unsigned int
Perfect_Hash_9::hash (const char *str, size_t len)
{
  static const unsigned char asso_values[] =
    {
      27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
      27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
      27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
      27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
      27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
      27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
      27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
      27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
      27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
      27, 27, 27, 27, 27,  0, 27, 27, 27, 27,
      27,  0, 27, 27, 27, 27, 27, 27, 27, 27,
      10, 27, 27, 27, 27,  0, 27, 27,  5, 27,
      27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
      27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
      27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
      27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
      27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
      27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
      27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
      27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
      27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
      27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
      27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
      27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
      27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
      27, 27, 27, 27, 27, 27
    };
  return len + asso_values[static_cast<unsigned char>(str[4])];
}

const struct word_type_9 *
Perfect_Hash_9::in_word_set (const char *str, size_t len)
{
  enum
    {
      TOTAL_KEYWORDS = 8,
      MIN_WORD_LENGTH = 9,
      MAX_WORD_LENGTH = 19,
      MIN_HASH_VALUE = 9,
      MAX_HASH_VALUE = 26
    };

  static const struct word_type_9 wordlist[] =
    {
      {"",{}}, {"",{}}, {"",{}}, {"",{}}, {"",{}}, {"",{}},
      {"",{}}, {"",{}}, {"",{}},
      {"read_some", 
            [](lua_State* L) -> int {
                rawgetp(L, LUA_REGISTRYINDEX, &tls_socket_read_some_key);
                return 1;
            }},
      {"write_some", 
            [](lua_State* L) -> int {
                rawgetp(L, LUA_REGISTRYINDEX, &tls_socket_write_some_key);
                return 1;
            }},
      {"",{}}, {"",{}}, {"",{}}, {"",{}},
      {"set_server_name", 
            [](lua_State* L) -> int {
#if (EMILUA_CONFIG_USE_STANDALONE_ASIO && !defined(ASIO_USE_WOLFSSL)) || (!EMILUA_CONFIG_USE_STANDALONE_ASIO && !defined(BOOST_ASIO_USE_WOLFSSL))
                lua_pushcfunction(L, tls_socket_set_server_name);
#else // (EMILUA_CONFIG_USE_STANDALONE_ASIO && !defined(ASIO_USE_WOLFSSL)) || (!EMILUA_CONFIG_USE_STANDALONE_ASIO && !defined(BOOST_ASIO_USE_WOLFSSL))
                lua_pushcfunction(L, throw_enosys);
#endif // (EMILUA_CONFIG_USE_STANDALONE_ASIO && !defined(ASIO_USE_WOLFSSL)) || (!EMILUA_CONFIG_USE_STANDALONE_ASIO && !defined(BOOST_ASIO_USE_WOLFSSL))
                return 1;
            }},
      {"server_handshake", 
            [](lua_State* L) -> int {
                rawgetp(L, LUA_REGISTRYINDEX, &socket_server_handshake_key);
                return 1;
            }},
      {"",{}}, {"",{}}, {"",{}},
      {"set_verify_mode", 
            [](lua_State* L) -> int {
                lua_pushcfunction(L, tls_socket_set_verify_mode);
                return 1;
            }},
      {"set_verify_depth", 
            [](lua_State* L) -> int {
                lua_pushcfunction(L, tls_socket_set_verify_depth);
                return 1;
            }},
      {"",{}}, {"",{}},
      {"set_verify_callback", 
            [](lua_State* L) -> int {
                lua_pushcfunction(L, tls_socket_set_verify_callback);
                return 1;
            }},
      {"",{}},
      {"client_handshake", 
            [](lua_State* L) -> int {
                rawgetp(L, LUA_REGISTRYINDEX, &socket_client_handshake_key);
                return 1;
            }}
    };

  if (len <= MAX_WORD_LENGTH && len >= MIN_WORD_LENGTH)
    {
      unsigned int key = hash (str, len);

      if (key <= MAX_HASH_VALUE)
        {
          const char *s = wordlist[key].name;

          if (*str == *s && !strcmp (str + 1, s + 1))
            return &wordlist[key];
        }
    }
  return 0;
}

} // namespace
} // namespace emilua::gperf::detail
#line 1 "../src/tls.cpp"
// Copyright (c) 2020, 2022, 2024 Vinícius dos Santos Oliveira
// SPDX-License-Identifier: MIT OR BSL-1.0

#line 19 "../src/tls.cpp"


namespace emilua {

extern unsigned char tls_dial_bytecode[];
extern std::size_t tls_dial_bytecode_size;

char tls_key;
char tls_context_mt_key;
char tls_socket_mt_key;

#line 121 "../src/tls.cpp"


static int extract_host(lua_State* L)
{
    luaL_checktype(L, 1, LUA_TSTRING);

    std::string_view host = tostringview(L, 1);

    {
        auto idx = host.rfind(':');
        if (idx == std::string_view::npos) {
            push(L, std::errc::invalid_argument, "arg", 1);
            return lua_error(L);
        }

        std::string_view port = host.substr(idx + 1);
        host.remove_suffix(port.size() + 1);
    }

    push(L, host);
    return 1;
}

static int tls_context_new(lua_State* L)
{
    luaL_checktype(L, 1, LUA_TSTRING);
    auto key = tostringview(L, 1);
    auto method = 
#line 2421 "libemilua.so.0.11.5.p/bc2_tls.cpp"
 ::emilua::gperf::detail::make_optional(::emilua::gperf::detail::Perfect_Hash_1::in_word_set((key).data(), (key).size()))
#line 174 "../src/tls.cpp"
;
    if (!method) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }
    try {
        auto ctx = std::make_shared<asio::ssl::context>(*method);

        auto c = static_cast<std::shared_ptr<asio::ssl::context>*>(
            lua_newuserdata(L, sizeof(std::shared_ptr<asio::ssl::context>))
        );
        rawgetp(L, LUA_REGISTRYINDEX, &tls_context_mt_key);
        setmetatable(L, -2);
        new (c) std::shared_ptr<asio::ssl::context>{std::move(ctx)};
        return 1;
    } catch (const asio_system_error& e) {
        push(L, static_cast<std::error_code>(e.code()));
        return lua_error(L);
    }
}

#line 1256 "../src/tls.cpp"


static int tls_context_mt_index(lua_State* L)
{
    auto key = tostringview(L, 2);
    return 
#line 2452 "libemilua.so.0.11.5.p/bc2_tls.cpp"
 ::emilua::gperf::detail::value_or(::emilua::gperf::detail::Perfect_Hash_6::in_word_set((key).data(), (key).size()), [](lua_State* L) -> int {
            push(L, errc::bad_index, "index", 2);
            return lua_error(L);
        })
#line 1387 "../src/tls.cpp"
(L);
}

static int tls_socket_new(lua_State* L)
{
    lua_settop(L, 2);

    auto tcp_s = static_cast<asio::ip::tcp::socket*>(lua_touserdata(L, 1));
    if (!tcp_s || !lua_getmetatable(L, 1)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }
    rawgetp(L, LUA_REGISTRYINDEX, &ip_tcp_socket_mt_key);
    if (!lua_rawequal(L, -1, -2)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }

    std::shared_ptr<asio::ssl::context>* c = nullptr;

    switch (lua_type(L, 2)) {
    default:
        push(L, std::errc::invalid_argument, "arg", 2);
        return lua_error(L);
    case LUA_TNIL:
        rawgetp(L, LUA_REGISTRYINDEX, &default_tls_context_key);
        c = static_cast<std::shared_ptr<asio::ssl::context>*>(
            lua_touserdata(L, -1));
        if (!c) {
            auto ctx = std::make_shared<asio::ssl::context>(
                asio::ssl::context::tlsv13);
            asio_error_code ec;
            ctx->set_default_verify_paths(ec);
            if (ec) {
                push(L, ec);
                return lua_error(L);
            }

            lua_pushlightuserdata(L, &default_tls_context_key);
            c = static_cast<std::shared_ptr<asio::ssl::context>*>(
                lua_newuserdata(L, sizeof(std::shared_ptr<asio::ssl::context>))
            );
            rawgetp(L, LUA_REGISTRYINDEX, &tls_context_mt_key);
            setmetatable(L, -2);
            new (c) std::shared_ptr<asio::ssl::context>{std::move(ctx)};
            lua_rawset(L, LUA_REGISTRYINDEX);
        }
        break;
    case LUA_TUSERDATA:
        c = static_cast<std::shared_ptr<asio::ssl::context>*>(
            lua_touserdata(L, 2)
        );
        if (!c || !lua_getmetatable(L, 2)) {
            push(L, std::errc::invalid_argument, "arg", 2);
            return lua_error(L);
        }
        rawgetp(L, LUA_REGISTRYINDEX, &tls_context_mt_key);
        if (!lua_rawequal(L, -1, -2)) {
            push(L, std::errc::invalid_argument, "arg", 2);
            return lua_error(L);
        }
        break;
    }

    auto s = static_cast<TlsSocket*>(lua_newuserdata(L, sizeof(TlsSocket)));
    rawgetp(L, LUA_REGISTRYINDEX, &tls_socket_mt_key);
    setmetatable(L, -2);
    new (s) TlsSocket{*tcp_s, *c};

    lua_pushnil(L);
    setmetatable(L, 1);
    tcp_s->asio::ip::tcp::socket::~socket();

    return 1;
}

template<asio::ssl::stream_base::handshake_type HANDSHAKE>
static int socket_handshake(lua_State* L)
{
    auto vm_ctx = get_vm_context(L).shared_from_this();
    auto current_fiber = vm_ctx->current_fiber();
    EMILUA_CHECK_SUSPEND_ALLOWED(*vm_ctx, L);

    auto s = static_cast<TlsSocket*>(lua_touserdata(L, 1));
    if (!s || !lua_getmetatable(L, 1)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }
    rawgetp(L, LUA_REGISTRYINDEX, &tls_socket_mt_key);
    if (!lua_rawequal(L, -1, -2)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }

    auto cancel_slot = set_default_interrupter(L, *vm_ctx);

    s->async_handshake(
        HANDSHAKE,
        asio::bind_cancellation_slot(cancel_slot, asio::bind_executor(
            vm_ctx->strand_using_defer(),
            [vm_ctx,current_fiber](const asio_error_code& ec) {
                auto opt_args = vm_context::options::arguments;
                vm_ctx->fiber_resume(
                    current_fiber,
                    hana::make_set(
                        vm_context::options::auto_detect_interrupt,
                        hana::make_pair(opt_args, hana::make_tuple(ec))));
            }
        )));

    return lua_yield(L, 0);
}

static int tls_socket_read_some(lua_State* L)
{
    lua_settop(L, 2);

    auto vm_ctx = get_vm_context(L).shared_from_this();
    auto current_fiber = vm_ctx->current_fiber();
    EMILUA_CHECK_SUSPEND_ALLOWED(*vm_ctx, L);

    auto s = static_cast<TlsSocket*>(lua_touserdata(L, 1));
    if (!s || !lua_getmetatable(L, 1)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }
    rawgetp(L, LUA_REGISTRYINDEX, &tls_socket_mt_key);
    if (!lua_rawequal(L, -1, -2)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }

    auto bs = static_cast<byte_span_handle*>(lua_touserdata(L, 2));
    if (!bs || !lua_getmetatable(L, 2)) {
        push(L, std::errc::invalid_argument, "arg", 2);
        return lua_error(L);
    }
    rawgetp(L, LUA_REGISTRYINDEX, &byte_span_mt_key);
    if (!lua_rawequal(L, -1, -2)) {
        push(L, std::errc::invalid_argument, "arg", 2);
        return lua_error(L);
    }

    auto cancel_slot = set_default_interrupter(L, *vm_ctx);

    s->async_read_some(
        asio::buffer(bs->data.get(), bs->size),
        asio::bind_cancellation_slot(cancel_slot, asio::bind_executor(
            vm_ctx->strand_using_defer(),
            [vm_ctx,current_fiber,buf=bs->data](
                const asio_error_code& ec, std::size_t bytes_transferred
            ) {
                boost::ignore_unused(buf);
                vm_ctx->fiber_resume(
                    current_fiber,
                    hana::make_set(
                        vm_context::options::auto_detect_interrupt,
                        hana::make_pair(
                            vm_context::options::arguments,
                            hana::make_tuple(ec, bytes_transferred))));
            }
        ))
    );

    return lua_yield(L, 0);
}

static int tls_socket_write_some(lua_State* L)
{
    lua_settop(L, 2);

    auto vm_ctx = get_vm_context(L).shared_from_this();
    auto current_fiber = vm_ctx->current_fiber();
    EMILUA_CHECK_SUSPEND_ALLOWED(*vm_ctx, L);

    auto s = static_cast<TlsSocket*>(lua_touserdata(L, 1));
    if (!s || !lua_getmetatable(L, 1)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }
    rawgetp(L, LUA_REGISTRYINDEX, &tls_socket_mt_key);
    if (!lua_rawequal(L, -1, -2)) {
        push(L, std::errc::invalid_argument, "arg", 1);
        return lua_error(L);
    }

    auto bs = static_cast<byte_span_handle*>(lua_touserdata(L, 2));
    if (!bs || !lua_getmetatable(L, 2)) {
        push(L, std::errc::invalid_argument, "arg", 2);
        return lua_error(L);
    }
    rawgetp(L, LUA_REGISTRYINDEX, &byte_span_mt_key);
    if (!lua_rawequal(L, -1, -2)) {
        push(L, std::errc::invalid_argument, "arg", 2);
        return lua_error(L);
    }

    auto cancel_slot = set_default_interrupter(L, *vm_ctx);

    s->async_write_some(
        asio::buffer(bs->data.get(), bs->size),
        asio::bind_cancellation_slot(cancel_slot, asio::bind_executor(
            vm_ctx->strand_using_defer(),
            [vm_ctx,current_fiber,buf=bs->data](
                const asio_error_code& ec, std::size_t bytes_transferred
            ) {
                boost::ignore_unused(buf);
                vm_ctx->fiber_resume(
                    current_fiber,
                    hana::make_set(
                        vm_context::options::auto_detect_interrupt,
                        hana::make_pair(
                            vm_context::options::arguments,
                            hana::make_tuple(ec, bytes_transferred))));
            }
        ))
    );

    return lua_yield(L, 0);
}

#line 1840 "../src/tls.cpp"


static int tls_socket_mt_index(lua_State* L)
{
    auto key = tostringview(L, 2);
    return 
#line 2686 "libemilua.so.0.11.5.p/bc2_tls.cpp"
 ::emilua::gperf::detail::value_or(::emilua::gperf::detail::Perfect_Hash_9::in_word_set((key).data(), (key).size()), [](lua_State* L) -> int {
            push(L, errc::bad_index, "index", 2);
            return lua_error(L);
        })
#line 1903 "../src/tls.cpp"
(L);
}

void init_tls(lua_State* L)
{
    lua_pushlightuserdata(L, &tls_key);
    {
        lua_createtable(L, /*narr=*/0, /*nrec=*/3);

        lua_pushliteral(L, "context");
        {
            lua_createtable(L, /*narr=*/0, /*nrec=*/1);

            lua_pushliteral(L, "new");
            lua_pushcfunction(L, tls_context_new);
            lua_rawset(L, -3);
        }
        lua_rawset(L, -3);

        lua_pushliteral(L, "socket");
        {
            lua_createtable(L, /*narr=*/0, /*nrec=*/1);

            lua_pushliteral(L, "new");
            lua_pushcfunction(L, tls_socket_new);
            lua_rawset(L, -3);
        }
        lua_rawset(L, -3);

        lua_pushliteral(L, "dial");
        int res = luaL_loadbuffer(
            L, reinterpret_cast<char*>(tls_dial_bytecode),
            tls_dial_bytecode_size, nullptr);
        assert(res == 0); boost::ignore_unused(res);
        lua_pushcfunction(L, extract_host);
        {
            rawgetp(L, LUA_REGISTRYINDEX, &ip_key);
            lua_getfield(L, -1, "tcp");
            lua_getfield(L, -1, "dial");
            lua_insert(L, -3);
            lua_pop(L, 2);
        }
        lua_pushcfunction(L, tls_socket_new);
        lua_call(L, 3, 1);
        lua_rawset(L, -3);
    }
    lua_rawset(L, LUA_REGISTRYINDEX);

    lua_pushlightuserdata(L, &tls_context_mt_key);
    {
        lua_createtable(L, /*narr=*/0, /*nrec=*/3);

        lua_pushliteral(L, "__metatable");
        lua_pushliteral(L, "tls.context");
        lua_rawset(L, -3);

        lua_pushliteral(L, "__index");
        lua_pushcfunction(L, tls_context_mt_index);
        lua_rawset(L, -3);

        lua_pushliteral(L, "__gc");
        lua_pushcfunction(L, finalizer<std::shared_ptr<asio::ssl::context>>);
        lua_rawset(L, -3);
    }
    lua_rawset(L, LUA_REGISTRYINDEX);

    lua_pushlightuserdata(L, &tls_socket_mt_key);
    {
        lua_createtable(L, /*narr=*/0, /*nrec=*/3);

        lua_pushliteral(L, "__metatable");
        lua_pushliteral(L, "tls.socket");
        lua_rawset(L, -3);

        lua_pushliteral(L, "__index");
        lua_pushcfunction(L, tls_socket_mt_index);
        lua_rawset(L, -3);

        lua_pushliteral(L, "__gc");
        lua_pushcfunction(L, finalizer<TlsSocket>);
        lua_rawset(L, -3);
    }
    lua_rawset(L, LUA_REGISTRYINDEX);

    rawgetp(L, LUA_REGISTRYINDEX, &var_args__retval1_to_error__key);

    lua_pushlightuserdata(L, &socket_client_handshake_key);
    lua_pushvalue(L, -2);
    rawgetp(L, LUA_REGISTRYINDEX, &raw_error_key);
    lua_pushcfunction(L, socket_handshake<asio::ssl::stream_base::client>);
    lua_call(L, 2, 1);
    lua_rawset(L, LUA_REGISTRYINDEX);

    lua_pushlightuserdata(L, &socket_server_handshake_key);
    lua_insert(L, -2);
    rawgetp(L, LUA_REGISTRYINDEX, &raw_error_key);
    lua_pushcfunction(L, socket_handshake<asio::ssl::stream_base::server>);
    lua_call(L, 2, 1);
    lua_rawset(L, LUA_REGISTRYINDEX);

    lua_pushlightuserdata(L, &tls_socket_read_some_key);
    rawgetp(L, LUA_REGISTRYINDEX,
            &var_args__retval1_to_error__fwd_retval2__key);
    rawgetp(L, LUA_REGISTRYINDEX, &raw_error_key);
    lua_pushcfunction(L, tls_socket_read_some);
    lua_call(L, 2, 1);
    lua_rawset(L, LUA_REGISTRYINDEX);

    lua_pushlightuserdata(L, &tls_socket_write_some_key);
    rawgetp(L, LUA_REGISTRYINDEX,
            &var_args__retval1_to_error__fwd_retval2__key);
    rawgetp(L, LUA_REGISTRYINDEX, &raw_error_key);
    lua_pushcfunction(L, tls_socket_write_some);
    lua_call(L, 2, 1);
    lua_rawset(L, LUA_REGISTRYINDEX);
}

} // namespace emilua
